C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - ARRSTO                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           COMPUTES THE MAXIMUM COLUMN REACHABLE AND THE MAXIMUM DIMENSIONS   +
C           OF THE WORKING ARRAYS USED IN THE FOLLOWING TWELVE SUBROUTINES     +
C           PERFORMING SCALAR EXTRAPOLATION ALGORITHMS.                        +
C           THE COMPUTATION IS MADE USING THE INPUT ARGUMENT NBC THAT          +
C           REPRESENTS THE MAXIMUM NUMBER OF CALLS OF EACH EXTRAPOLATION       +
C           SUBROUTINE.                                                        +
C                                                                              +
C            I  SUBROUTINE ALGORITHM                                           +
C            -  ---------- ---------                                           +
C                                                                              +
C            1  EALGO      E-ALGORITHM.                                        +
C            2  EPSRHA     SIMULTANEOUS EPSILON-ALGORITHM, FIRST AND SECOND    +
C                          GENERALIZATIONS OF THE EPSILON-ALGORITHM AND        +
C                          RHO-ALGORITHM.                                      +
C            3  EPSRHO     EPSILON-ALGORITHM, FIRST AND SECOND GENERALIZATIONS +
C                          OF THE EPSILON-ALGORITHM AND RHO-ALGORITHM.         +
C            4  GTRAN      G-TRANSFORMATION.                                   +
C            5  IDELTA     ITERATED DELTA2 PROCESS.                            +
C            6  LEVINT     LEVIN'S TRANSFORMS.                                 +
C            7  OMEGA      w-ALGORITHM.                                        +
C            8  OVERHO     OVERHOLT PROCESS.                                   +
C            9  QDALGO     qd-ALGORITHM.                                       +
C           10  RICHAR     RICHARDSON PROCESS.                                 +
C           11  RSALGO     rs-ALGORITHM.                                       +
C           12  THETA      THETA-ALGORITHM.                                    +
C                                                                              +
C   USAGE:                                                                     +
C           CALL ARRSTO (NBC, MAXC, IA, IER)                                   +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           NBC    - INPUT INTEGER. IT REPRESENTS THE MAXIMUM NUMBER OF CALLS  +
C                    OF THE EXTRAPOLATION SUBROUTINE. IT MUST NOT BE LESS      +
C                    THAN 3.                                                   +
C                                                                              +
C           MAXC   - OUTPUT INTEGER VECTOR THAT REPRESENTS THE MAXIMUM COLUMN  +
C                    REACHABLE WITH THE VALUE OF NBC.                          +
C                    THE COMPONENT MAXC(I) IS RELATED TO THE I-TH SUBROUTINE   +
C                    ACCORDING WITH THE ORDER GIVEN IN THE DESCRIPTION.        +
C                                                                              +
C           IA     - OUTPUT INTEGER VECTOR THAT REPRESENTS THE STORAGE AMOUNT  +
C                    FOR THE WORKING ARRAYS OF THE DIFFERENT SUBROUTINES.      +
C                    THE COMPONENTS ARE ASSIGNED IN THE FOLLOWING WAY:         +
C                                                                              +
C                    SUBROUTINE EALGO:                                         +
C                    IA(1)   E(0:MAXCOL)              REAL                     +
C                    IA(2)   GINIT(2*MAXCOL)          REAL                     +
C                    IA(3)   G(MAXCOL*(MAXCOL+1))     REAL                     +
C                                                                              +
C                    SUBROUTINE EPSRHA:                                        +
C                    IA(4)   EPSVEC(0:3,0:MAXCOL)     REAL                     +
C                    IA(5)   ABVEC(0:3,2*MAXSIN)      REAL                     +
C                    IA(6)   XAUX(3,0:MAXCOL)         REAL                     +
C                    IA(7)   INDSAV(0:3,MAXSIN)       INTEGER                  +
C                                                                              +
C                    SUBROUTINE EPSRH0:                                        +
C                    IA(8)   EPSVEC(0:MAXCOL)         REAL                     +
C                    IA(9)   ABVEC(2*MAXSIN)          REAL                     +
C                    IA(10)  XAUX(0:MAXCOL)           REAL                     +
C                    IA(11)  INDSAV(MAXSIN)           INTEGER                  +
C                                                                              +
C                    SUBROUTINE GTRAN:                                         +
C                    IA(12)  GVEC(0:2*MAXCOL-1)       REAL                     +
C                    IA(13)  RSVEC(0:2*(MAXCOL-1))    REAL                     +
C                                                                              +
C                    SUBROUTINE IDELTA:                                        +
C                    IA(14)  XVEC(0:2*MAXCOL)         REAL                     +
C                                                                              +
C                    SUBROUTINE LEVINT:                                        +
C                    IA(15)  VECN(0:MAXCOL)           REAL                     +
C                    IA(16)  VECD(0:MAXCOL)           REAL                     +
C                                                                              +
C                    SUBROUTINE OMEGA:                                         +
C                    IA(17)  OMEVEC(0:MAXCOL)         REAL                     +
C                                                                              +
C                    SUBROUTINE OVERHO:                                        +
C                    IA(18)  V(0:MAXCOL)              REAL                     +
C                                                                              +
C                    SUBROUTINE QDALGO:                                        +
C                    IA(19)  QEVEC(0:2*MAXCOL-1)      REAL                     +
C                                                                              +
C                    SUBROUTINE RICHAR:                                        +
C                    IA(20)  T(0:2*MAXCOL)            REAL                     +
C                    IA(21)  XAUX(0:2*MAXCOL)         REAL                     +
C                                                                              +
C                    SUBROUTINE RSALGO:                                        +
C                    IA(22)  RSVEC(0:2*MAXCOL-1)      REAL                     +
C                                                                              +
C                    SUBROUTINE THETA:                                         +
C                    IA(23)  THVEC(0:2*MAXCOL)        REAL                     +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 1400    NBC VALUE LESS THAN 3.                      +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  FOR THE EPSILON-ALGORITHM, ITS GENERALIZATIONS AND FOR THE          +
C          RHO-ALGORITHM THE VALUE OF MAXSIN (MAXIMUM NUMBER OF SINGULARITIES) +
C          IS SET EQUAL TO THE MAXIMUM COLUMN INDEX MINUS 3, IF THE MAXIMUM    +
C          COLUMN INDEX IS GREATER THAN 3, AND TO ONE, OTHERWISE.              +
C       -  IN THE CALLING PROCEDURE, THE DIMENSIONS FOR THE VECTOR ARGUMENTS   +
C          MUST BE:                                                            +
C           MAXC(12)                                                           +
C           IA(23)                                                             +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE ARRSTO (NBC, MAXC, IA, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, NBC
      INTEGER IA(1), MAXC(1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      INTEGER MAXSIN
C
C                        ... EXECUTABLE STATEMENTS
C
      IER = 0
      IF (NBC .LT. 3) THEN
         IER = 1400
         RETURN
      END IF
C
C                        ... EALGO
C
      MAXC(1) = NBC - 1
      IA(1)   = MAXC(1) + 1
      IA(2)   = 2 * MAXC(1)
      IA(3)   = MAXC(1) * IA(1)
C
C                        ... EPSRHA
C
      MAXC(2) = NBC - 1
      IF (MOD(MAXC(2),2) .NE. 0) MAXC(2) = MAXC(2) - 1
      IA(4)   = 4 * (MAXC(2) + 1)
      IF (MAXC(2) .GT. 3) THEN
         MAXSIN  = MAXC(2) - 3
      ELSE
         MAXSIN  = 1
      END IF
      IA(5)   = 8 * MAXSIN
      IA(6)   = 3 * (MAXC(2) + 1)
      IA(7)   = 4 * MAXSIN
C
C                        ... EPSRHO
C
      MAXC(3) = MAXC(2)
      IA(8)   = MAXC(3) + 1
      IA(9)   = 2 * MAXSIN
      IA(10)  = IA(8)
      IA(11)  = MAXSIN
C
C                        ... GTRAN
C
      MAXC(4) = NBC / 2
      IA(12)  = 2 * MAXC(4)
      IA(13)  = IA(12) -1
C
C                        ... IDELTA
C
      MAXC(5) = (NBC - 1)/2
      IA(14)  = 2 * MAXC(5) + 1
C
C                        ... LEVINT
C
      MAXC(6) = NBC - 1
      IA(15)  = MAXC(6) + 1
      IA(16)  = IA(15)
C
C                        ... OMEGA
C
      MAXC(7) = MAXC(2)
      IA(17)  = MAXC(7) + 1
C
C                        ... OVERHO
C
      MAXC(8) = NBC - 2
      IA(18)  = MAXC(8) + 1
C
C                        ... QDALGO
C
      MAXC(9) = (NBC - 1)/2
      IA(19)  = 2 * MAXC(9)
C
C                        ... RICHAR
C
      MAXC(10) = NBC - 2
      IA(20)   = 2 * MAXC(10) + 1
      IA(21)   = IA(20)
C
C                        ... RSALGO
C
      MAXC(11) = MAXC(9)
      IA(22)   = 2 * MAXC(11)
C
C                        ... THETA
C
      MAXC(12) = 2 * (NBC - 1)/3
      IF (MOD(MAXC(12),2) .NE. 0) MAXC(12) = MAXC(12) - 1
      IA(23)   = 2 * MAXC(12) + 1
C
      RETURN
      END
