C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - BETABOR                                              +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           COMPUTES THE ARRAY beta  OF DIMENSIONS p  x p  AND DETERMINES      +
C                                  k                k    k                     +
C           IF IT IS OR NOT A SINGULAR MATRIX.                                 +
C                                                                              +
C   USAGE:                                                                     +
C           CALL BETABOR (A, BETA, IN, IP, IR, WK, EPS, INDS)                  +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL SQUARE MATRIX OF DIMENSION (IR,IR).            +
C                                                                              +
C           BETA   - OUTPUT REAL MATRIX OF DIMENSION (IR,2*IR).                +
C                    IF THE MATRIX beta IS NON SINGULAR (INDS.EQ.0) ITS FIRST  +
C                                      k                        -1             +
C                    COLUMNS CONTAIN, IN OUTPUT, THE MATRIX beta .             +
C                                                               k              +
C                                                                              +
C           IN     - INPUT INTEGER DIMENSION OF THE SQUARE MATRIX A  .         +
C                                                                  k           +
C                                                                              +
C           IP     - INPUT INTEGER DIMENSION p OF THE SQUARE MATRIX beta .     +
C                                             k                         k      +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRICES A AND BETA.                                  +
C                                                                              +
C           WK     - WORKING REAL VECTOR OF DIMENSION IR.                      +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           INDS   - OUTPUT INTEGER VALUE.                                     +
C                    IF INDS = 0 THEN beta  IS NON-SINGULAR.                   +
C                                         k                                    +
C                    IF INDS = 1 THEN beta  IS SINGULAR.                       +
C                                         k                                    +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       - SUBROUTINES    INVERS, RCPROD                                        +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C        - IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRIX A       +
C          AND BETA MUST BE THE SAME.                                          +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE BETABOR (A, BETA, IN, IP, IR, WK, EPS, INDS)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IN, INDS, IP, IR
      DOUBLE PRECISION EPS
      DOUBLE PRECISION A(IR,1), BETA(IR,1), WK(1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER I, J
C
C                        ... EXECUTABLE STATEMENTS
C
      INDS = 0
C                                                     -1
C                        ... COMPUTES THE VALUE OF v  A   AND  
C                                                   k  k  
C                            STORES IT IN BETA-ARRAY
C
      CALL RCPROD (A, IN+1, 1, IR, A, 1, 1, IR, BETA, 1, 1, IR,
     *             IP, IN, IN, 0.0D0, 1.0D0, 0, WK)
C
C                                                     -1
C                        ... COMPUTES THE VALUE OF (v A  ) u   AND 
C                                                    k k    k
C                            STORES IT IN BETA-ARRAY
C
      CALL RCPROD (BETA, 1, 1, IR, A, 1, IN+1, IR, BETA, 1, 1, IR,
     *             IP, IN, IP, 0.0D0, 1.0D0, 0, WK)
C
C                                                                 -1
C                        ... COMPUTES THE VALUE OF beta  = a  - v A  u 
C                                                      k    k    k k  k
C
      DO 20 J = 1, IP
         DO 10 I = 1, IP
            BETA(I,J) = A(IN+I,IN+J) - BETA(I,J)
  10  CONTINUE
  20  CONTINUE
C                                                    -1
C                        ... COMPUTES THE INVERSE beta   OR THE SINGULARITY OF
C                            beta .                   k
C                                k
C
      IF (IP .EQ. 1) THEN
         IF (DABS(BETA(1,1)) .LT. EPS) THEN
            INDS = 1
         ELSE
            BETA(1,1) = 1.0D0 / BETA(1,1)
         END IF
      ELSE
         CALL INVERS (BETA, IP, IR, EPS, INDS)
      END IF
      RETURN
      END      
