C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - BLBORD                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           COMPUTES THE SOLUTION z    OF THE BORDERED SYSTEM                  +
C                                  k+1                                         +
C           A    z    = d  IN TERMS OF THE SOLUTION z  OF THE PREVIOUS         +
C            k+1  k+1    k+1                         k                         +
C           SYSTEM A  z  = d  .                                                +
C                   k  k    k                                                  +
C                                                                              +
C   USAGE:                                                                     +
C           CALL BLBORD (A, D, IR, Z, BETA, WK, EPS, INIT, IER)                +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL SQUARE MATRIX OF DIMENSION (IR,IR).            +
C                                                                              +
C           D      - INPUT REAL VECTOR OF DIMENSION IR.                        +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX A.                                             +
C                                                                              +
C           Z      - OUTPUT REAL VECTOR OF DIMENSION IR CONTAINING             +
C                    AFTER THE k-TH CALL, IN THE FIRST k COMPONENTS,           +
C                    THE SOLUTION z  OF A z = d .                              +
C                                  k     k k   k                               +
C                                                                              +
C           BETA   - WORKING REAL MATRIX OF DIMENSION (IR,2*IR).               +
C                    IF THE MATRIX beta IS NON SINGULAR (INDS.EQ.0) ITS FIRST  +
C                                      k                        -1             +
C                    COLUMNS CONTAIN, IN OUTPUT, THE MATRIX beta .             +
C                                                               k              +
C                                                                              +
C           WK     - WORKING REAL VECTOR OF DIMENSION IR.                      +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE         +
C                    FIRST CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED        +
C                    TO 1 BY THE SUBROUTINE DURING THE FIRST CALL.             +
C                    FOR A NEW APPLICATION OF THE METHOD INIT MUST SET         +
C                    AGAIN TO ZERO.                                            +
C                                                                              +
C           IER    - INPUT/OUTPUT INDEX WARNING/ERROR.                         +
C                    IER =  200    CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER =  300    ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO NUMBER_OF_CALLS.      +
C                    IER = 2100    SINGULAR MATRIX IN THE ALGORITHM.           +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C        - SUBROUTINES     BETABOR, RCPROD                                     +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION. THE VECTORS Z AND WK, AND THE MATRIX BETA      +
C          MUST NOT BE MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS      +
C          OF THE SUBROUTINE.                                                  +
C        - IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRICES A     +
C          AND BETA MUST BE THE SAME.                                          +
C        - WHEN THE SUBROUTINE RETURNS AN IER VALUE OF 2100, THE USER MUST     +
C          GIVE AN ADDITIONAL ROW AND COLUMN OF THE A-ARRAY AND SET IER TO     +
C          ZERO BEFORE THE NEXT CALL.                                          +
C        - IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=NUMBER_OF_CALLS) ARE:               +
C           A(IR,IR)                                                           +
C           D(IR)                                                              +
C           Z(IR)                                                              +
C           BETA(IR,2*IR)                                                      +
C           WK(IR)                                                             +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE BLBORD (A, D, IR, Z, BETA, WK, EPS, INIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IR
      DOUBLE PRECISION EPS
      DOUBLE PRECISION A(IR,1), BETA(IR,1), D(1), Z(1), WK(1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES AND
C                            CONSTANTS
C
      DOUBLE PRECISION COEFF, MONE, ONE, ZERO
      INTEGER I, IFLAGM, IN, INDS, INK, IP, J
      SAVE IFLAGM, INK, IP
C
C                        ... SPECIFICATIONS FOR PARAMETER CONSTANTS
C
      PARAMETER (ZERO=0.0D0, ONE=1.0D0, MONE=-1.0D0)
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER    = 0
         INIT   = 1
         IFLAGM = 0
         INK    = 1
         IF (DABS(A(1,1)) .LT. EPS) THEN
            INK    = INK + 1
            IFLAGM = 1
            IER    = 2100
            RETURN
         END IF
         Z(1)   = D(1) / A(1,1)
         A(1,1) = 1.0D0 / A(1,1)
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C                        ... IFLAGM = 1 IF PREVIOUS A  ARRAY IS SINGULAR
C                                                    1
C
      IF (IFLAGM .EQ. 1) THEN
C
C                        ... COPIES THE NEW A ARRAY IN BETA ARRAY.
C                                            1
C
         DO 20 I = 1, INK
            DO 10 J = 1, INK
               BETA (I,J) = A(I,J)
   10       CONTINUE
   20    CONTINUE
C
C                        ... CONTROLS THE SINGULARITY OF A
C                                                         1
C
         CALL INVERS (BETA, INK, IR, EPS, INDS)
C
C                        ... IF A IS SINGULAR, RETURNS TO CALLING PROGRAM 
C                                1
C                            FOR ADDITIONAL ROW AND COLUMN
C
         IF (INDS .EQ. 1) THEN
            INK = INK + 1
            IF (INK .GT. IR) THEN
               IER = 300
            ELSE
               IER = 2100
            END IF
         ELSE
C
C                        ... IF A IS NON-SINGULAR, COMPUTES z  AND RETURNS
C                                1                           1
C                            TO CALLING PROGRAM
C
C                                           -1
C                        ... COPIES BETA = A  ARRAY IN A  ARRAY.
C                                           1           1
C
            DO 40 I = 1, INK
               DO 30 J = 1, INK
                  A(I,J) = BETA (I,J) 
  30           CONTINUE
  40        CONTINUE
            IFLAGM = 0
C
C                        ... COMPUTES z  
C                                      1                        
C
            CALL RCPROD (A, 1, 1, IR, D, 1, 1, IR, Z, 1, 1, IR,
     *                   INK, INK, 1, ZERO, ONE, 0, WK)
         END IF
         RETURN
      END IF
C
C                        ... THE A  ARRAY IS NON-SINGULAR
C                                 1
C
         INK = INK + 1
         IF (INK .GT. IR) THEN
            IER  = 300
            RETURN
         END IF
C                        ... IFLAGM = 0 IF beta  IS NON-SINGULAR.
C                                              k                  
C
      IF (IFLAGM .EQ. 0) IP  = 1
C
C                        ... IFLAGM = 2 IF beta  IS SINGULAR.
C                                              k                               
C
      IF (IFLAGM .EQ. 2) IP  = IP + 1
C
C                        ... COMPUTES THE NEW beta
C                                                 k
C
      IN  = INK - IP
      CALL BETABOR (A, BETA, IN, IP, IR, WK, EPS, INDS)
C
C                        ... IF beta IS SINGULAR, RETURNS TO CALLING PROGRAM 
C                                   k
C                            FOR ADDITIONAL ROW AND COLUMN
C
      IF (INDS .EQ. 1) THEN
         IFLAGM = 2 
         IER = 2100
         RETURN
      END IF
C
C                                                               -1
C                        ... IF beta IS NON-SINGULAR, COMPUTES A    AND z
C                                   k                           k+1      z+1
C
      IFLAGM = 0 
C                                       -1
C                        ... COPIES beta  IN A-ARRAY IN a  
C                                       k                k
C
      DO 60 I = 1, IP
         DO 50 J = 1, IP
            A(IN+I,IN+J) = BETA(I,J)
   50    CONTINUE
   60 CONTINUE
C
C                                                     -1
C                        ... COMPUTES THE VALUE OF - A   u   AND 
C                                                     k   k    
C                            STORES IT IN A-ARRAY IN u  
C                                                     k
C
      CALL RCPROD (A, 1, 1, IR, A, 1, IN+1, IR, A, 1, IN+1, IR,
     *             IN, IN, IP, ZERO, MONE, 1, WK)
C
C                        ... COMPUTES THE VALUE OF v  z  AND 
C                                                   k  k    
C                            STORES IT IN Z-VECTOR (ELEMENTS n +1 TO n + p )
C                                                             k       k   k
C
      CALL RCPROD (A, IN+1, 1, IR, Z, 1, 1, IR, Z, IN+1, 1, IR,
     *             IP, IN, 1, ZERO, ONE, 0, WK)
C
C                        ... COMPUTES THE VALUE OF  f - v  z  AND 
C                                                    k   k  k 
C                            STORES IT IN Z-VECTOR (ELEMENTS n +1 TO n + p )
C                                                             k       k   k
C
      DO 70 I = 1, IP
         Z(IN+I) = D(IN+I) - Z(IN+I)
   70 CONTINUE
C
C                                                       -1
C                        ... COMPUTES THE VALUE OF  beta  v  AND 
C                                                       k  k    
C                            STORES IT IN A-ARRAY IN v  
C                                                     k
C
      CALL RCPROD (A, IN+1, IN+1, IR, A, IN+1, 1, IR, A, IN+1, 1, IR,
     *             IP, IP, IN, ZERO, ONE, 1, WK)
C
C                                                        -1    -1
C                        ... COMPUTES THE VALUE OF - beta  v  A  AND 
C                                                        k  k  k  
C                            STORES IT IN A-ARRAY IN v 
C                                                     k
C
      CALL RCPROD (A, IN+1, 1, IR, A, 1, 1, IR, A, IN+1, 1, IR, 
     *             IP, IN, IN, ZERO, MONE, 0, WK)
C
C                                                   -1    -1        -1   -1
C                        ... COMPUTES THE VALUE OF A  +  A   u  beta  v A   
C                                                   k     k   k     k  k k
C                            AND STORES IT IN A-ARRAY 
C
      CALL RCPROD (A, 1, IN+1, IR, A, IN+1, 1, IR, A, 1, 1, IR,
     *             IN, IP, IN, ONE, ONE, 0, WK)
C                                                     -1        -1
C                        ... COMPUTES THE VALUE OF - A   u  beta   AND 
C                                                     k   k     k
C                            STORES IT IN A-ARRAY IN u  
C                                                     k
C
      IF (IP .EQ. 1) THEN
         DO 80 I = 1, IN
            A(I,IN+1) = A(I,IN+1) * A(IN+1,IN+1)
   80    CONTINUE
C
C                                                       -1        -1
C                        ... COMPUTES THE VALUE OF z - A   u  beta   (f - v z )
C                                                   k   k   k     k    k   k k
C                            AND STORES IT IN Z-VECTOR (ELEMENTS 1 TO n )
C                                                                      k
C
         DO 90 I =1, IN
            Z(I) = Z(I) + A(I,IN+1) * Z(IN+1)
   90    CONTINUE
C                                                      -1       
C                        ... COMPUTES THE VALUE OF beta   (f - v z ) AND
C                                                      k    k   k k
C                            STORES IT IN Z-VECTOR (ELEMENTS n + 1 TO n + p )
C                                                             k        k   k
C
         Z(IN+1) = Z(IN+1) * A(IN+1,IN+1) 
      ELSE  
C                                                     -1        -1
C                        ... COMPUTES THE VALUE OF - A   u  beta   AND 
C                                                     k   k     k
C                            STORES IT IN A-ARRAY IN u  
C                                                     k
C
         CALL RCPROD (A, 1, IN+1, IR, A, IN+1, IN+1, IR, A, 1, IN+1, IR,
     *                IN, IP, IP, ZERO, ONE, 0, WK)
C
C                                                       -1        -1
C                        ... COMPUTES THE VALUE OF z - A   u  beta   (f - v z )
C                                                   k   k   k     k    k   k k
C                            AND STORES IT IN Z-VECTOR (ELEMENTS 1 TO n )
C                                                                      k
C
         CALL RCPROD (A, 1, IN+1, IR, Z, IN+1, 1, IR, Z, 1, 1, IR,
     *                IN, IP, 1, ONE, ONE, 0, WK)
C                                                      -1       
C                        ... COMPUTES THE VALUE OF beta   (f - v z ) AND
C                                                      k    k   k k
C                            STORES IT IN Z-VECTOR (ELEMENTS n + 1 TO n + p )
C                                                             k        k   k
C
         CALL RCPROD (A, IN+1, IN+1, IR, Z, IN+1, 1, IR, Z, IN+1, 1, IR,
     *                IP, IP, 1, ZERO, ONE, 1, WK)
      END IF
C
C                                                     -1
C                        ... SOLUTION z    AND ARRAY A    COMPUTED
C                                      k+1            k+1
C
      RETURN
      END      
