C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - COMPOS                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           COMPOSITE SEQUENCE TRANSFORMATION OF RANK k (k<=9).                +
C           THE USER MAY CHOOSE BETWEEN THE FOLLOWING T  TRANSFORMATIONS       +
C           (I = 1, ..., 9)                            I                       +
C                                                                              +
C            I = 1  EPSILON-ALGORITHM                                          +
C            I = 2  RHO-ALGORITHM WITH x  = n                                  +
C                                       n                                      +
C            I = 3  THETA-ALGORITHM                                            +
C            I = 4  RICHARDSON PROCESS WITH x  = S    -  S                     +
C                                            n    n+1     n                    +
C            I = 5  FIRST GENER. EPSILON-ALGORITHM WITH x  = 1 + 1/(n+1)       +
C                                                        n                     +
C            I = 6  SECOND GENER. EPSILON-ALGORITHM WITH x  = 1 + 1/(n+1)      +
C                                                         n                    +
C            I = 7  ITERATED DELTA2 PROCESS                                    +
C            I = 8  OVERHOLT PROCESS                                           +
C            I = 9  LEVIN'S t-TRANSFORM                                        +
C                                                                              +
C            THE SUBROUTINE CONSIDERS ONLY THE TRANSFORMATIONS T  WITH A       +
C                                                               I              +
C            POSITIVE VALUE OF THE CORRESPONDING COMPONENT IOR(I).             +
C            THUS THE USER CHOOSES THE DESIRED TRANSFORMATIONS (FOR EXAMPLE    +
C            T  , T  , T AND T ), AFTER HE SETS IOR(1), IOR(3), IOR(5) AND     +
C             1    3    5     7                                                +
C            IOR(7) TO THE APPROPRIATE ORDER CHOSEN (FOR EXAMPLE IOR(1) = 3,   +
C            IOR(3) = 1, IOR(5) = 4 AND IOR(7)=2 FOR USING THE THETA-ALG. AS   +
C            THE FIRST TRANSFORMATION, THE ITERATED DELTA2 PROCESS AS THE      +
C            SECOND ONE, AND SO ON). ALL THE OTHER COMPONENTS OF IOR MUST BE   +
C            SET TO A NON-POSITIVE INTEGER NUMBER. IN THIS EXAMPLE THE RANK k  +
C            IS EQUAL TO 4.                                                    +
C            THE COMPOSITION IS MADE USING THE E-ALGORITHM AND IT STARTS AT    +
C            THE 4-TH CALL (BECAUSE THE THETA-ALGORITHM GIVES THE FIRST VALUE  +
C            OF THE COLUMN 2 AFTER 4 CALLS).                                   +
C                                                                              +
C   USAGE:                                                                     +
C           CALL COMPOS (E, NBC, INIT, EPS, SINIT, IOR, NTR, RES,              +
C              NDIGIT, IER)                                                    +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           E      - OUTPUT REAL VECTOR THAT CONTAINS, AFTER THE m-TH CALL     +
C                    OF THE SUBROUTINE, THE LAST COMPUTED ROW OF THE E-ARRAY:  +
C                                                                              +
C                     - IF m < 4          UNDEFINED                            +
C                                                                              +
C                                          (m-4)  (m-3)       (0)              +
C                     - IF 4 =< m <= k+3  E    , E     ,..., E                 +
C                                          0      1           m-4              +
C                                                                              +
C                                          (m-4)  (m-3)       (m-3-k)          +
C                     - IF m > k+3        E    , E     ,..., E                 +
C                                          0      1           k-1              +
C                                                                              +
C                    WHERE k IS THE RANK OF THE TRANSFORMATION (NUMBER OF      +
C                    TRANSFORMATIONS CONSIDERED BY THE SUBROUTINE ACCORDING    +
C                    TO THE USER CHOICE).                                      +
C                    IN THE CALLING PROCEDURE THE DIMENSION OF E MUST BE       +
C                    (0:IE), WITH IE GREATER OR EQUAL TO k-1.                  +
C                                                                              +
C           NBC    - INPUT INTEGER. IT REPRESENTS THE MAXIMUM NUMBER OF CALLS  +
C                    OF THIS SUBROUTINE AS GIVEN IN THE CALLING PROGRAM. THE   +
C                    SUBROUTINE USES THIS VALUE TO SET THE DIMENSIONS OF THE   +
C                    ARRAYS USED IN THE DIFFERENT TRANSFORMATIONS AND TO       +
C                    DEFINE THE INDEX OF THE LAST COLUMN FOR ALL THE           +
C                    TRANSFORMATIONS. ITS VALUE MUST NOT BE LESS THAN 3.       +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY THE  +
C                    SUBROUTINE DURING THE FIRST CALL.                         +
C                    FOR A NEW COMPOSITION THE USER MUST SET AGAIN INIT TO     +
C                    ZERO.                                                     +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           SINIT  - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S    BEFORE THE m-TH CALL OF THE SUBROUTINE.           +
C                        m-1                                                   +
C                    IT CONTAINS AFTER THE m-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                    S            IF  m < 4                                    +
C                     m-1                                                      +
C                                                                              +
C                     (0)                                                      +
C                    E            IF  4 <= m <= k + 3                          +
C                     m-4                                                      +
C                                                                              +
C                     (m-3-k)                                                  +
C                    E            IF  m > k + 3                                +
C                     k-1                                                      +
C                                                                              +
C                    THE SUBROUTINE, AFTER THE 4th CALL RETURNS THE SEQUENCE   +
C                                                                              +
C                     (0)   (0)   (0)         (0)    (1)    (2)                +
C                    E   , E   , E   , ... , E    , E    , E   , ...           +
C                     0     1     2           k-1    k-1    k-1                +
C                                                                              +
C           IOR    - INPUT INTEGER VECTOR. EACH COMPONENT IOR(I)(FOR I=1,..,9) +
C                    CORRESPONDS TO THE I-TH TRANSFORMATION ACCORDING TO THE   +
C                    ORDER GIVEN IN THE DESCRIPTION OF THE SUBROUTINE.         +
C                    IF IOR(I)<=0 THE CORRESPONDING TRANSFORMATION IS NOT USED +
C                    IF IOR(I)>0  IT CONTAINS THE ORDER THAT THE USER ASSIGN   +
C                                 TO THE I-TH TRANSFORMATION. THUS, FOR        +
C                                 EXAMPLE, IF THE USER WANTS TO USE FIRST THE  +
C                                 THIRD TRANSFORMATION (THAT IS THE THETA      +
C                                 ALGORITHM), HE MUST SET IOR(3)=1.            +
C                                                                              +
C           NTR    - INPUT INTEGER. IT REPRESENTS THE RANK k, THAT IS THE      +
C                    NUMBER OF DIFFERENT TRANSFORMATIONS THAT THE USER WANTS   +
C                    TO USE IN THE COMPOSITION.                                +
C                                                                              +
C           RES    - OUTPUT REAL VECTOR CONTAINING THE RESULT GIVEN BY EACH    +
C                    SELECTED TRANSFORMATION AFTER THE m-TH CALL. IN THE       +
C                    CALLING PROCEDURE THE DIMENSION MUST BE AT MOST RES(NTR). +
C                    IT IS ALWAYS DEFINED (ALSO WHEN m < 4).                   +
C                    THE RESULTS OF THE SELECTED TRANSFORMATIONS ARE STORED IN +
C                    THE COMPONENTS OF THE VECTOR ACCORDING TO THE ORDER GIVEN +
C                    BY THE USER. FOR EXAMPLE, IF IOR(1)=3, THEN RES(3) IS THE +
C                    VALUE COMPUTED BY THE EPSILON ALGORITHM. IN GENERAL, THE  +
C                    VALUE RES(IOR(I)) CONTAINS THE RESULT OF THE TRANSF. T .  +
C                                                                          I   +
C                                                                              +
C           NDIGIT - INPUT INTEGER VALUE. IT REPRESENTS THE NUMBER OF DIGITS   +
C                    THAT THE USER ALLOWS TO LOSE, DUE TO THE CANCELLATION     +
C                    ERROR, IN THE RHO-ALGORITHM, IN THE EPSILON-ALGORITHM AND +
C                    IN ITS GENERALIZATIONS (SEE THE SUBROUTINE EPSRHA FOR     +
C                    ADDITIONAL INFORMATIONS). IF NONE OF THESE TRANSFORM.     +
C                    ENTERS IN THE COMPOSITION, THEN ITS VALUE IS NOT USED.    +
C                    WHEN EPSRHA IS CALLED, THE VALUE OF MAXSIN (MAXIMUM       +
C                    NUMBER OF SINGULARITIES) IS SET EQUAL TO THE MAXIMUM      +
C                    COLUMN INDEX MINUS 3.                                     +
C                                                                              +
C           IER    - OUTPUT INTEGER VECTOR CONTAINING THE WARNING/ERROR.       +
C                    THIS VECTOR CONTAINS:                                     +
C                    IN IER(I), I= 1, 9 VALUE OF IER FOR THE TRANSFORM. T      +
C                                                                        I     +
C                                       ACCORDING TO THE ORDER GIVEN IN THE    +
C                                       DESCRIPTION (SEE THE MEANING OF IER IN +
C                                       EACH SUBROUTINE ).                     +
C                    IN IER(10)         VALUE OF IER FOR THE SUBROUTINE EALGO. +
C                    IN IER(11)         VALUE OF IER FOR THE SUBROUTINE COMPOS.+
C                                                                              +
C                    FOR IER(11) WE HAVE:                                      +
C                      IER(11) = 200     CALL OF THE SUBROUTINE WITH A NON     +
C                                        ZERO IER VALUE.                       +
C                      IER(11) = 600     NO TRANSFORMATION HAVE BEEN CHOSEN    +
C                                        (NTR=0 THAT IS THE RANK k IS ZERO).   +
C                      IER(11) = 700     THE INPUT VALUE NTR DON'T CORRESPOND  +
C                                        TO THE NUMBER OF TRANSFORMATIONS      +
C                                        CHOSEN (THAT IS THE TRANSFORMATIONS   +
C                                        WITH IOR(I) GREATER THAN ZERO AND     +
C                                        LESS THAN 10).                        +
C                      IER(11) = 800     TWO COMPONENTS OF IOR ARE EQUAL.      +
C                      IER(11) = 900     IN THE VECTOR IOR, SOME INTEGER       +
C                                        BETWEEN 1 AND NTR IS MISSING.         +
C                      IER(11) = 1000    NON ZERO IER VALUE IN SOME SUBROUTINE +
C                                        CALLED BY COMPOS.                     +
C                      IER(11) = 1300    THE SIZE OF THE WORKING VECTOR IS     +
C                                        GREATER THAN THE ALLOWED VALUE. THE   +
C                                        SUBROUTINE INCSTO MUST BE USED IN THE +
C                                        CALLING PROGRAM.                      +
C                      IER(11) = 1400    VALUE OF NBC LESS THAN 3.             +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       -  SUBROUTINES  ARRSTO, EALGO, EPSRHA, IDELTA, INCSTO,                 +
C                       LEVINT, OVERHO, RICHAR, THETA                          +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION AND THE VECTORS E, IOR AND RES MUST NOT BE   +
C          MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE           +
C          SUBROUTINE.                                                         +
C       -  THE SUBROUTINE WORKS ONLY WITH A VALUE OF NBC NOT GREATER THAN 30.  +
C          THIS IS DUE TO THE PREFIXED MAXIMUM DIMENSION OF THE WORKING ARRAY  +
C          AWRK IN THE COMMON STATEMENT. TO INCREASE THIS VALUE THE USER MUST  +
C          USE, IN THE CALLING PROGRAM, THE SUBROUTINE INCSTO AS EXPLAINED     +
C          IN ITS DESCRIPTION.                                                 +
C       -  TO OBTAIN THE FIRST RESULT OF THE COMPOSITION, THE USER MUST CALL   +
C          THIS SUBROUTINE AT LEAST 4 TIMES.                                   +
C       -  IN THE CALLING PROCEDURE, THE MINIMAL DIMENSIONS FOR THE VECTOR     +
C          ARGUMENTS ARE (NTR=k):                                              +
C           E(0:NTR-1)                                                         +
C           IOR(9)                                                             +
C           RES(NTR)                                                           +
C           IER(11)                                                            +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE COMPOS (E, NBC, INIT, EPS, SINIT, IOR, NTR, RES,
     *       NDIGIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER INIT, NBC, NDIGIT, NTR
      DOUBLE PRECISION EPS, SINIT
      INTEGER IOR(1), IER(1)
      DOUBLE PRECISION E(0:1), RES(1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES AND
C                            CONSTANTS
C
      INTEGER I, IAMAX, IK, INDM, INI, ITRAN, J, NBMTR, NTR1, MAXSIN
      INTEGER*4 FIXDIM, IDIM
      DOUBLE PRECISION B, SSAV, XINIT, VINIT, THINIT, GINITL,
     *       LINIT, TINIT, EINIT
      SAVE B, INDM, INI, ITRAN, NBMTR, MAXSIN, SSAV
C
C                        ... SPECIFICATIONS FOR PARAMETER
C
      PARAMETER (FIXDIM = 1000)
C
C                        ... SPECIFICATIONS FOR LOCAL ARRAYS
C
      INTEGER NSING(0:3), IA(23), IN(11), INITS(7), MAXC(12)
      DOUBLE PRECISION ET(9,0:8), EPSINI(0:3), XINITV(3), GINIT(16)
      SAVE INITS, IN, MAXC
C
C                        ... COMMON AND EQUIVALENCE STATEMENTS
C
      DOUBLE PRECISION AWRK(FIXDIM)
      INTEGER*4 IWRK(FIXDIM*2)
      EQUIVALENCE (AWRK(1),IWRK(1))
      COMMON /WRKSPR/ AWRK
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         INDM   = -1
         INIT   = 1
         NBMTR  = 9
         B      = 1.0D0
         ITRAN  = 2
         DO 10 I = 1, 11
            IER(I) = 0
            IF (I .LE. 7) INITS(I) = 0
   10    CONTINUE
C
         IF (NBC .LT. 3) THEN
            IER(11) = 1400
            RETURN
         END IF
         IF (NTR .EQ. 0) THEN
            IER(11) = 600
            RETURN
         END IF
C
C                        ... COMPUTE THE TOTAL NUMBER OF TRANSFORMATIONS
C                            NTR1 TO BE CONSIDERED (ACCORDING TO THE USER
C                            CHOICE) AND CONTROL IF IT IS EQUAL TO NTR.
C
         NTR1 = 0
         DO 20 I = 1, NBMTR
            IF (IOR(I) .GT. 0 .AND. IOR(I) .LE. NBMTR)
     *         NTR1 = NTR1 + 1
   20    CONTINUE
         IF (NTR1 .NE. NTR) THEN
            IER(11) = 700
            RETURN
         END IF
C
C                        ... CONTROL IF TWO VALID COMPONENTS
C                            OF IOR ARE EQUAL.
C
         DO 40 I = 1, NBMTR-1
            IF (IOR(I) .GT. 0 .AND. IOR(I) .LE. NBMTR) THEN
               DO 30 J = I+1, NBMTR
                  IF (IOR(I) .EQ. IOR(J)) THEN
                     IER(11) = 800
                     RETURN
                  END IF
   30          CONTINUE
            END IF
   40    CONTINUE
C
C                        ... CONTROL IF ALL THE VALUES FOR THE ORDER
C                            ARE STORED IN IOR.
C
         DO 60 I = 1, NTR
            DO 50 J = 1, NBMTR
               IF (I .EQ. IOR(J)) GOTO 60
   50       CONTINUE
         IER(11) = 900
         RETURN
   60    CONTINUE
C
C                        ... DEFINE THE MAXIMUM WORKING STORAGE
C
         IDIM = FIXDIM
         CALL INCSTO (IDIM)
C
C                        ... COMPUTE THE STORAGE OCCUPATION
C
         CALL ARRSTO (NBC, MAXC, IA, I)
C
C                        ... DEFINE THE INDEXES FOR THE WORKING VECTOR
C
         IN(1)  = IA(4)  + 1
         IN(2)  = IN(1)  + IA(5)
         IN(3)  = IN(2)  + IA(6)
         IN(4)  = IN(3)  + IA(23)
         IN(5)  = IN(4)  + IA(20)
         IN(6)  = IN(5)  + IA(21)
         IN(7)  = IN(6)  + IA(14)
         IN(8)  = IN(7)  + IA(18)
         IN(9)  = IN(8)  + IA(15)
         IN(10) = IN(9)  + IA(16)
         IN(11) = IN(10) + (NTR-1) * (NBC-3)
C
C                        ... CONTROL THE MAXIMUM ARRAY SIZE
C
         INI   = IN(11) * 2 - 1
         IAMAX = IN(11) + (IA(7)+1)/2 - 1
         IF (IAMAX .GT. IDIM) THEN
            IER(11) = 1300
            RETURN
         END IF
C
         IF (MAXC(2) .GT. 3) THEN
            MAXSIN = MAXC(2) - 3
         ELSE
            MAXSIN = 1
         END IF
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      DO 70 I = 1, 11
         IF (IER(I) .NE. 0) THEN
            IER(11) = 200
            RETURN
         END IF
   70 CONTINUE
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1)
C
      INDM = INDM+1
      IF (IOR(1) .GT. 0 .OR. IOR(2) .GT. 0 .OR. IOR(5) .GT. 0
     *   .OR. IOR(6) .GT. 0) THEN
C
C                        ... CALL EPSRHA
C
         DO 80 I = 0, 3
            EPSINI(I) = SINIT
   80    CONTINUE
         XINITV(1) = 1.0D0 + 1.0D0/DBLE(INDM+1)
         XINITV(2) = XINITV(1)
         XINITV(3) = DBLE(INDM)
         CALL EPSRHA (AWRK(1), MAXC(2), INITS(1), EPS, EPSINI,
     *      NDIGIT, MAXSIN, NSING, IWRK(INI), AWRK(IN(1)),
     *      AWRK(IN(2)), XINITV, IER)
         IER(5) = IER(2)
         IER(6) = IER(3)
         IER(2) = IER(4)
         IF (IOR(1) .GT. 0) THEN
            IF (IER(1) .NE. 0) THEN
               IER(11) = 1000
               RETURN
            ELSE
               RES(IOR(1)) = EPSINI(0)
            END IF
         END IF
         IF (IOR(5) .GT. 0) THEN
            IF (IER(5) .NE. 0) THEN
               IER(11) = 1000
               RETURN
            ELSE
               RES(IOR(5)) = EPSINI(1)
            END IF
         END IF
         IF (IOR(6) .GT. 0) THEN
            IF (IER(6) .NE. 0) THEN
               IER(11) = 1000
               RETURN
            ELSE
               RES(IOR(6)) = EPSINI(2)
            END IF
         END IF
         IF (IOR(2) .GT. 0) THEN
            IF (IER(2) .NE. 0) THEN
               IER(11) = 1000
               RETURN
            ELSE
               RES(IOR(2)) = EPSINI(3)
            END IF
         END IF
      END IF
C
      IF (IOR(3) .GT. 0) THEN
C
C                        ... CALL THETA
C
         THINIT = SINIT
         CALL THETA (AWRK(IN(3)), MAXC(12), INITS(2), EPS,
     *      THINIT, IER(3))
         IF (IER(3) .NE. 0) THEN
            IER(11) = 1000
            RETURN
         END IF
         RES(IOR(3)) = THINIT
      END IF
C
      IF (IOR(4) .GT. 0) THEN
         IF (INDM .NE. 0) THEN
C
C                        ... CALL RICHAR
C
            TINIT = SSAV
            XINIT = SINIT - SSAV
            CALL RICHAR (AWRK(IN(4)), MAXC(10), INITS(3), EPS,
     *         TINIT, AWRK(IN(5)), XINIT, IER(4))
            IF (IER(4) .NE. 0) THEN
               IER(11) = 1000
               RETURN
            END IF
            RES(IOR(4)) = TINIT
         ELSE
            RES(IOR(4)) = SINIT
         END IF
         SSAV = SINIT
      END IF
C
      IF (IOR(7) .GT. 0) THEN
C
C                        ... CALL IDELTA
C
         XINIT = SINIT
         CALL IDELTA (AWRK(IN(6)), MAXC(5), INITS(4), EPS,
     *      XINIT, IER(7))
         IF (IER(7) .NE. 0) THEN
            IER(11) = 1000
            RETURN
         END IF
         RES(IOR(7)) = XINIT
      END IF
C
      IF (IOR(8) .GT. 0) THEN
C
C                        ... CALL OVERHO
C
         VINIT = SINIT
         CALL OVERHO (AWRK(IN(7)), MAXC(8), INITS(5), EPS,
     *      VINIT, IER(8))
         IF (IER(8) .NE. 0) THEN
            IER(11) = 1000
            RETURN
         END IF
         RES(IOR(8)) = VINIT
      END IF
C
      IF (IOR(9) .GT. 0) THEN
C
C                        ... CALL LEVINT
C
         LINIT  = SINIT
         CALL LEVINT (AWRK(IN(8)), AWRK(IN(9)), MAXC(6), INITS(6),
     *        EPS, LINIT, GINITL, B, ITRAN, IER(9))
         IF (IER(9) .NE. 0) THEN
            IER(11) = 1000
            RETURN
         END IF
         RES(IOR(9)) = LINIT
      END IF
C
C                        ... CONTROL THE BEGINNING OF THE COMPOSITION.
C
      IF (INDM .LT. 3) RETURN
C
C                        ... COMPOSITION OF THE TRANSFORMATIONS CHOSEN
C                            BY THE USER
C
      J  = INDM - 2
      IK = MIN0(INDM-3, NTR-1)
      DO 90 I = 1, NTR
         ET(I,IK) = RES(I)
   90 CONTINUE
      IF (J.EQ.1 .OR. J.EQ.2) THEN
         GINIT(1) = ET(2,J-1) - ET(1,J-1)
      ELSE IF (J .LE. NTR) THEN
         DO 100 I = 1, J - 2
            GINIT(I) = ET(I+1,J-1) - ET(I,J-1)
  100    CONTINUE
         DO 110 I = 0, J - 1
            GINIT(I+J-1) = ET(J,I) - ET(J-1,I)
  110    CONTINUE
      ELSE
         DO 120 I = 1, NTR-1
            GINIT(I) = ET(I+1,IK) - ET(I,IK)
  120    CONTINUE
      END IF
      EINIT = ET(1,IK)
C
C                        ... CALL EALGO
C
      CALL EALGO (E, NTR-1, INITS(7), EPS, EINIT, GINIT,
     *   AWRK(IN(10)), IER(10))
      IF (IER(10) .NE. 0) THEN
         IER(11) = 1000
         RETURN
      END IF
C
C                        ... SET THE RESULT VALUE
C
      SINIT = EINIT
C
      RETURN
      END
