C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - CRPA                                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION VIA EITHER THE CRPA OR ONE OF        +
C           ITS TWO VARIANTS.                                                  +
C                                                                              +
C   USAGE:                                                                     +
C           CALL CRPA (E, MAXCOL, INIT, EPS, XINIT, ZINIT, IR, IP,             +
C                  IALG, IER)                                                  +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           E      - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSIONS OF E MUST BE (IR,0:IE), WITH IE GREATER OR     +
C                    EQUAL TO MAXCOL. THE COLUMNS OF THIS MATRIX CONTAIN,      +
C                    AFTER THE k-TH CALL, THE VECTORS OF THE LAST COMPUTED ROW +
C                    OF THE e-ARRAY:                                           +
C                                                                              +
C                                          (k-1)  (k-2)       (0)              +
C                     - IF k <= MAXCOL+1  e    , e     ,..., e                 +
C                                          0      1           k-1              +
C                                                                              +
C                                          (k-1)  (k-2)       (k-1-MAXCOL)     +
C                     - IF k > MAXCOL+1   e    , e     ,..., e                 +
C                                          0      1           MAXCOL           +
C                                                                              +
C           MAXCOL - INPUT/OUTPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN  +
C                    OF THE e-ARRAY THAT THE USER WANTS TO COMPUTE             +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF e   ). MAXCOL MUST NOT BE GREATER     +
C                                         k         (n)                        +
C                    THAN IP SINCE ALL THE VECTORS e   WITH k > IP ARE ZERO    +
C                                                   k                          +
C                    (BECAUSE z  , ..., z  ARE LINEARLY DEPENDENT).            +
C                              1         k                                     +
C                    IF MAXCOL IS GREATER THAN IP, ITS VALUE IS CHANGED TO IP  +
C                    BY THE SUBROUTINE.                                        +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           XINIT  - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VECTOR      +
C                    OF x    BEFORE THE k-TH CALL OF THE SUBROUTINE.           +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                     (0)                                                      +
C                    e            IF  k <= MAXCOL+1                            +
C                     k-1                                                      +
C                                                                              +
C                     (k-1-MAXCOL)                                             +
C                    e            IF  k >  MAXCOL+1                            +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)   (0)   (0)         (0)      (1)      (2)            +
C                    e   , e   , e   , ... , e      , e      , e      , ...    +
C                     0     1     2           MAXCOL   MAXCOL   MAXCOL         +
C                                                                              +
C           ZINIT  - INPUT REAL MATRIX. IN THE CALLING PROCEDURE THE           +
C                    DIMENSIONS OF ZINIT MUST BE (IR,IE), WITH IE GREATER OR   +
C                    EQUAL TO MAXCOL.                                          +
C                    BEFORE THE k-TH CALL OF THE SUBROUTINE, FOR k >= 2, THE   +
C                    (k-1)-TH COLUMN OF ZINIT MUST CONTAIN THE VECTOR z   .    +
C                                                                      k-1     +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRICES E AND ZINIT, AND DIMENSION OF THE VECTOR XINIT.  +
C                    IR MUST BE GREATER OR EQUAL TO IP.                        +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF ROWS FOR   +
C                    THE MATRICES E AND ZINIT, AND DIMENSION OF THE VECTOR     +
C                    XINIT).                                                   +
C                                                                              +
C           IALG   - INPUT/OUTPUT INTEGER FLAG.                                +
C                    IF IALG .EQ. 0 THE SUBROUTINE PERFORMS THE CRPA.          +
C                    IF IALG .EQ. 1 THE SUBROUTINE PERFORMS ITS FIRST VARIANT. +
C                    IF IALG .EQ. 2 THE SUBROUTINE PERFORMS ITS SECOND VARIANT +
C                    IF IALG .LT. 0 .OR. IALG .GT. 2 THE SUBROUTINE PUTS       +
C                    IALG TO ZERO AND THUS IT PERFORMS THE CRPA.               +
C                                                                              +
C           IER    - INPUT/OUTPUT INDEX WARNING/ERROR.                         +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO THE VECTOR SPACE      +
C                                  DIMENSION.                                  +
C                    IER = 2000    MAXCOL IS GREATER THAN IP. MAXCOL IS SET    +
C                                  TO IP BY THE SUBROUTINE.                    +
C                                  IER MUST BE SET AGAIN TO ZERO BY THE USER   +
C                                  BEFORE THE NEXT CALL.                       +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE MATRICES E AND ZINIT MUST NOT BE MODIFIED  +
C          BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.        +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF e   (J <= MAXCOL), THE USER MUST CALL THIS   +
C                                  J                                           +
C          SUBROUTINE (J+1) TIMES.                                             +
C       -  IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRICES E AND +
C          ZINIT, AND THE DIMENSION OF THE VECTOR XINIT MUST BE THE SAME.      +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=IP) ARE:                            +
C           E(IP,0:MAXCOL)                                                     +
C           XINIT(IP)                                                          +
C           ZINIT(IP,MAXCOL)                                                   +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE CRPA (E, MAXCOL, INIT, EPS, XINIT, ZINIT, 
     *       IR, IP, IALG, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IALG, IER, INIT, IP, IR, MAXCOL
      DOUBLE PRECISION EPS
      DOUBLE PRECISION E(IR,0:1), XINIT(1), ZINIT(IR,1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      DOUBLE PRECISION SE, SX, TMP
      INTEGER I, INDM, J, M
      SAVE INDM
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER    = 0
         INDM   = 0
         INIT   = 1
C
         IF (IP .GT. IR) THEN
            IER = 300
            RETURN
         END IF
C
         IF (IALG .LT. 0 .OR. IALG .GT. 2) IALG = 0
C
         IF (MAXCOL .GT. IP) THEN
            MAXCOL = IP
            IER = 2000
         END IF
         DO 10 I = 1, IP
            E(I,0) = XINIT(I)
   10    CONTINUE
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         DO 20 I = 1, IP
            E(I,0) = XINIT(I)
   20    CONTINUE
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS -1)
C
      INDM = INDM+1
C
C                                                         (0)   (0)
C                        ... LOOP FOR THE COMPUTATION OF e   , e   , ....
C                                                         0     1
C
      M = MIN0(INDM,MAXCOL)
      DO 50 I = 0, M-1
C
C                        ... COMPUTATION OF THE SCALAR PRODUCTS INVOLVED
C                            IN THE RULE OF THE ALGORITHM
C
         SX = 0.0D0
         SE = 0.0D0
         DO 30 J = 1, IP
            SX = SX + ZINIT(J,I+1) * XINIT(J)
            SE = SE + ZINIT(J,I+1) * E(J,I)
   30    CONTINUE
         DO 40 J = 1, IP
            IF (IALG .EQ. 0) THEN
C
C                        ... CRPA
C
               IF (DABS(SX) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
               TMP = E(J,I) - SE * XINIT(J) / SX
            ELSE IF (IALG .EQ. 1) THEN
C
C                        ... FIRST VARIANT OF THE CRPA
C
               IF (DABS(SE) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
               TMP = SX * E(J,I) / SE - XINIT(J)
            ELSE
C
C                        ... SECOND VARIANT OF THE CRPA
C
               IF (DABS(SX-SE) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
               TMP = E(J,I) - (XINIT(J) - E(J,I)) * SE / (SX-SE)
            END IF
            E(J,I) = XINIT(J)
            XINIT(J) = TMP
   40    CONTINUE
   50 CONTINUE
C
C                        ... SETS THE RESULT VECTOR
C
      DO 60 J = 1, IP
         E(J,M) = XINIT(J)
   60 CONTINUE
C
      RETURN
      END
