C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - EPSVEC                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE VECTOR EPSILON-ALGORITHM WITH THE      +
C           NORMAL OR THE PARTICULAR RULES OF CORDELLIER ACCORDING TO THE      +
C           DESIRED PERMISSIBLE DEGREE OF CANCELLATION.                        +
C                                                                              +
C   USAGE:                                                                     +
C           CALL EPSVEC (EPSMAT, MAXCOL, INIT, EPS, EPSINI, IR,                +
C              IP, NDIGIT, MAXSIN, NSING, INDSAV, ABMAT, WG, SVAL, IER)        +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           EPSMAT - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSIONS OF EPSMAT MUST BE (IP,0:IE), WITH IE GREATER   +
C                    OR EQUAL TO MAXCOL (IF MAXCOL IS EVEN) OR TO (MAXCOL-1)   +
C                    (IF MAXCOL IS ODD). THIS MATRIX CONTAINS, AFTER THE k-TH  +
C                    CALL THE LAST COMPUTED BACKWARD DIAGONAL OF THE EPSILON   +
C                    SCHEME STORED BY COLUMNS:                                 +
C                                                                              +
C                                          (k-1)   (k-2)        (0)            +
C                     - IF k <= MAXCOL+1 eps   , eps    ,..., eps              +
C                                           0       1            k-1           +
C                                                                              +
C                                          (k-1)   (k-2)        (k-1-MAXCOL)   +
C                     - IF k > MAXCOL+1  eps   , eps    ,..., eps              +
C                                           0       1            MAXCOL        +
C                                                                              +
C           MAXCOL - INPUT/OUTPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN  +
C                    OF THE EPSILON SCHEME THAT THE USER WANTS TO COMPUTE.     +
C                                           (n)                                +
C                    (THE LOWER INDEX OF eps   ). IF MAXCOL IS ODD THE         +
C                                            m                                 +
C                    SUBROUTINE CHANGES ITS VALUE TO (MAXCOL-1).               +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           EPSINI - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VALUE       +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                       (0)                                                    +
C                    eps            IF  k <= MAXCOL+1, k ODD                   +
C                       k-1                                                    +
C                                                                              +
C                       (1)                                                    +
C                    eps            IF  k <  MAXCOL+1, k EVEN                  +
C                       k-2                                                    +
C                                                                              +
C                       (k-1-MAXCOL)                                           +
C                    eps            IF  k >  MAXCOL+1                          +
C                       MAXCOL                                                 +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                      (0)   (1)   (0)   (1)       (0)      (1)      (2)       +
C                    eps , eps , eps , eps ,..., eps    , eps    , eps   , ... +
C                       0     0     2     2         MAXCOL   MAXCOL   MAXCOL   +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRIX EPSMAT.                                            +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF ELEMENTS   +
C                    OF THE VECTORS EPSINI AND Y).                             +
C                                                                              +
C           NDIGIT - INPUT INTEGER VALUE. IT REPRESENTS THE NUMBER OF DIGITS   +
C                    THAT THE USER ALLOWS TO LOSE DUE TO THE CANCELLATION      +
C                    ERROR. IF FOR ANY INDEX k AND FOR ANY n THE RELATION      +
C                        (n+1)   (n)      (n)     -NDIGIT                      +
C                    ||eps   - eps ||/||eps ||< 10      HOLDS, WHERE ||.||     +
C                         k       k        k                                   +
C                    REPRESENTS THE NORM OF THE VECTOR I.E. THE SQUARE ROOT    +
C                    OF THE SCALAR PRODUCT OF THE VECTOR BY ITSELF, THEN THE   +
C                    RULES OF CORDELLIER SHALL BE USED TO COMPUTE THE VALUE    +
C                         (n-1)                                                +
C                    OF eps    ONE CALL AFTER. BUT IF, IN THE NEXT CALL, WE    +
C                          k+3      (n+2)   (n+1)      (n+1)     -NDIGIT       +
C                    ALSO HAVE  ||eps   - eps   ||/||eps  || < 10      THEN    +
C                                    k       k         k                       +
C                    THE SUBROUTINE STOPS DUE TO A NON ISOLATED SINGULARITY.   +
C                                                                              +
C           MAXSIN - INPUT INTEGER GIVING THE MAXIMUM NUMBER OF SINGULARITIES  +
C                    THAT THE SUBROUTINE CAN STORE.                            +
C                                                                              +
C           NSING  - OUTPUT INTEGER VALUE. IT REPRESENTS THE TOTAL NUMBER OF   +
C                    SINGULARITIES DETECTED IN ALL THE PRECEDING CALLS OF THE  +
C                    SUBROUTINE.                                               +
C                                                                              +
C           INDSAV - WORKING INTEGER VECTOR IN WHICH THE SUBROUTINE SAVES THE  +
C                    LOWER INDICES OF SINGULARITIES. IN THE CALLING PROCEDURE  +
C                    ITS DIMENSION MUST BE GREATER OR EQUAL TO MAXSIN.         +
C                                                                              +
C           ABMAT  - WORKING REAL MATRIX. IN THE FIRST MAXSIN COLUMNS THE      +
C                    SUBROUTINE SAVES THE VALUES OF W AND, IN THE SUBSEQUENT   +
C                    MAXSIN COLUMNS, THE VALUES OF N. IN THE CALLING PROCEDURE +
C                    THE LOWER DIMENSION OF ABMAT IS IP AND THE UPPER          +
C                    DIMENSION MUST BE GREATER OR EQUAL TO 2*MAXSIN.           +
C                                                                              +
C           WG     - WORKING REAL VECTOR OF DIMENSION 4*IP.                    +
C                                                                              +
C           SVAL   - WORKING REAL VECTOR TO STORE THE SQUARE OF THE NORM FOR   +
C                    THE VECTORS (W - C), (N - C) AND THE VALUE OF LAMBDA      +
C                    CORRESPONDING TO EACH SINGULARITY DETECTED.               +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO THE VECTOR SPACE      +
C                                  DIMENSION.                                  +
C                    IER = 1100    TOO MANY SINGULARITIES TO BE STORED.        +
C                                  INCREASE THE VALUE OF MAXSIN AND THE        +
C                                  DIMENSION OF THE VECTORS ABMAT AND INDSAV.  +
C                    IER = 1200    SINGULARITY NON ISOLATED. IMPOSSIBLE TO     +
C                                  CONTINUE.                                   +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       -  DOUBLE PRECISION FUNCTION  ENORM2                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION. THE MATRICES EPSMAT AND ABMAT, AND THE      +
C          VECTORS INDSAV, WG AND SVAL MUST NOT BE MODIFIED BY THE USER        +
C          BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.                    +
C                                    (0)                                       +
C       -  TO OBTAIN THE VALUE OF eps   (2*J <= MAXCOL) THE USER MUST CALL     +
C                                    2*J                                       +
C          THIS SUBROUTINE (2*J+1) TIMES.                                      +
C       -  IT IS UNUSEFULL TO ASSIGN MAXSIN A VALUE GREATER THAT (MAXCOL-3).   +
C          WHEN MAXCOL<=2, THE PARTICULAR RULES CANNOT BE USED.                +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=IP) ARE:                            +
C           EPSMAT(IP,0:MAXCOL)                                                +
C           EPSINI(IP)                                                         +
C           INDSAV(MAXSIN)                                                     +
C           ABMAT(IP,2*MAXSIN)                                                 +
C           WG(4*IP)                                                           +
C           SVAL(3*MAXSIN)                                                     +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE EPSVEC (EPSMAT, MAXCOL, INIT, EPS, EPSINI, IR, IP,
     *       NDIGIT, MAXSIN, NSING, INDSAV, ABMAT, WG, SVAL, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IP, IR, MAXCOL, MAXSIN, NDIGIT, NSING
      INTEGER INDSAV(1)
      DOUBLE PRECISION EPS 
      DOUBLE PRECISION EPSMAT(IR,0:1), EPSINI(1), WG(1), ABMAT(IR,1),
     *                 SVAL(1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      DOUBLE PRECISION ENORM2
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER II, LBM, LBM2, IP2, IP3, INDM, INDS, LISTBEG, LISTEND
      DOUBLE PRECISION RD, RDD, TMP, ANDIG
      LOGICAL*1 IAC, IBC, IEC, LOCINS, NEWINS
      SAVE LISTBEG, LISTEND, INDM, IP2, IP3,
     *     LOCINS, NEWINS, IEC, IAC, IBC, 
     *     ANDIG
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IF (IP .GT. IR) THEN
            IER = 300
            RETURN
         END IF
         IER    = 0                                                           
         INIT   = 1
         NSING  = 0
         ANDIG  = 10.0D0 ** (-FLOAT(NDIGIT))
         LOCINS = .FALSE.
         NEWINS = .FALSE.
         IEC    = .FALSE.
         IAC    = .FALSE.
         IBC    = .FALSE.
         INDM   = 0
         IP2    = IP * 2
         IP3    = IP * 3
C
C                        ... IF MAXCOL IS ODD IT IS MODIFIED TO
C                            (MAXCOL - 1).
C
         IF (MOD(MAXCOL,2) .NE. 0) MAXCOL = MAXCOL-1
C
C
         DO 10 II = 1, IP
            EPSMAT(II,0) = EPSINI(II)
   10    CONTINUE
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE BACKWARD DIAGONAL TO 
C                            BE COMPUTED AND THE LOWER INDEX m OF THE 
C                                                          (0)
C                            LAST VALUE OF THIS DIAGONAL eps    TO BE 
C                                                           m
C                            COMPUTED IN THE CALL.
C
      INDM = INDM + 1 
C
C                        ... IF INDM .GT. MAXCOL THEN THE ELEMENT 
C                               (NUMBER_OF_CALLS-MAXCOL+1)
C                            eps                          IS COMPUTED.
C                               MAXCOL
C
      IF (INDM .GT. MAXCOL) INDM = MAXCOL
C
C                        ... IF THERE IS NO LOCAL INSTABILITY PRESENT,
C                            RESET THE INDEXES LISTBEG AND LISTEND OF
C                            THE VECTOR INDSAV.
C
      IF (.NOT. LOCINS) THEN
         LISTBEG = 1
         LISTEND = 0
      END IF
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE 
C                            AUX1.
C
      DO 20 II = 1, IP
         WG(IP+II) = EPSINI(II)
   20 CONTINUE
C
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS
C                            OF THE ASCENDING DIAGONAL.
C
      DO 170 INDS = 0, INDM - 1
C  
C                        ... COMPUTE THE INVERSE OF THE DIFFERENCE
C                            VECTOR OF THE EPSILON-ALGORITHM FORMULA.
C
         DO 30 II = 1, IP
            WG(II) = WG(IP+II) - EPSMAT(II,INDS)
   30    CONTINUE
C  
C                        ... CONTROL FOR A NEW POINT OF INSTABILITY.
C
         RD  = ENORM2(WG, 1, IP, IP) 
         RDD = ENORM2(EPSMAT, INDS+1, IR, IP)
         IF (DABS(RDD) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         IF (DSQRT(RDD) .GT. EPS .AND. DSQRT(RD/RDD) .LT. ANDIG .AND. 
     *       .NOT.IEC .AND. INDS .LT. INDM-1 .AND. INDS .LT. MAXCOL-2) 
     *   THEN
            NEWINS  = .TRUE.
            NSING   = NSING + 1
            LISTEND = LISTEND+1
            IF (LISTEND .GT. MAXSIN) THEN
               IER = 1100
               RETURN
            END IF
         END IF 
C
         IF (LOCINS .OR. NEWINS) THEN
C
C                        ... WHEN THERE IS A LOCAL INSTABILITY PRESENT
C                            OR A NEW POINT OF INSTABILITY WE USE
C                            THE SINGULAR RULE OF THE ALGORITHM.
C
            IF (NEWINS) THEN
C   
C                        ... THERE IS A NEW POINT OF INSTABILITY ...
C
               IF (LOCINS) THEN
C  
C                        ... AND THERE IS A LOCAL INSTABILITY PRESENT.
C
C  
C                        ... IF INDS COINCIDES WITH THE FIRST ELEMENT
C                            STORED IN INDSAV THEN THERE IS A SINGULARITY 
C                            NON ISOLATED AND IT IS IMPOSSIBLE TO CONTINUE.
C
                  IF (INDS .EQ. INDSAV(LISTBEG)) THEN
                     IER = 1200
                     RETURN
                  END IF
               END IF
C  
C                        ... LOCAL INSTABILITY PRESENT IS TRUE.
C
               LOCINS = .TRUE.
C
C                        ... WE MUST CONSIDER THE COMPUTATIONS RELATED TO A!
C
               IAC    = .TRUE.
C  
C                        ... NEW POINT OF INSTABILITY TREATED!
C
               NEWINS = .FALSE.
C
C                        ... ADD THE CURRENT INDS TO THE END OF INDSAV.
C
               INDSAV(LISTEND) = INDS
            END IF
C   
C                        ... WHEN THE CONTROL ON A NEW POINT OF INSTABILITY
C                            IS MADE, THE CONTROL FOR THE COMPUTATION OF E
C                            IS PERFORMED.
C
            IF (IEC) THEN
C
C                        ... WE MUST COMPUTE E AND STORE IT IN AUX0.
C
               LBM  = LISTBEG + MAXSIN
               LBM2 = LISTBEG + 2*MAXSIN
               RD   = 1.0D0/SVAL(LBM) + 1.0D0/TMP - 1.0D0/SVAL(LISTBEG) 
     *               + SVAL(LBM2)
               IF (DABS(RD) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
               DO 40 II = 1, IP
                  WG(II) = (ABMAT(II,LBM)/SVAL(LBM) + 
     *               WG(IP2+II)/TMP - ABMAT(II,LISTBEG)/SVAL(LISTBEG) + 
     *               SVAL(LBM2) * EPSMAT(II,INDS-1)) / RD
   40          CONTINUE
C  
C                        ... E COMPUTED ACCORDING TO THE PARTICULAR RULES!
C
               IEC     = .FALSE.
C
C                        ... INCREASE THE POINTER ON THE VECTOR INDSAV.
C
               LISTBEG = LISTBEG + 1
C
C                        ... IF, IN THE VECTOR INDSAV, ALL THE STORED
C                            INDEX HAVE BEEN CONSIDERED, THERE IS NO
C                            LOCAL INSTABILITY PRESENT.
C
               IF (LISTBEG .GT. LISTEND) LOCINS = .FALSE.
C
            ELSE
C
C                        ... IF WE DON'T COMPUTE E, THE NORMAL RULE IS
C                            CONSIDERED.
C
               RD = ENORM2(WG, 1, IP, IP)
               IF (DABS(RD) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
               DO 50 II = 1, IP
                  WG(II) = WG(II) / RD
                  IF (INDS .NE. 0) WG(II) = WG(II) + EPSMAT(II,INDS-1)
   50          CONTINUE
            END IF
C
            IF (LOCINS) THEN
C   
C                        ... THERE IS A LOCAL INSTABILITY PRESENT ...
C
               IF (IAC) THEN
C
C                        ... AND WE MUST STORE THE VALUE OF W IN A.
C
                  IF (INDS .NE. 0) THEN
                     DO 60 II = 1, IP
                        ABMAT(II,LISTEND) = EPSMAT(II,INDS-1) 
   60                CONTINUE
                  ELSE
                     DO 70 II = 1, IP
                        ABMAT(II,LISTEND) = 0.0D0
   70                CONTINUE
                  END IF
C  
C                        ... COMPUTE THE VECTOR (W - C) AND ITS SQUARED NORM.
C
                  DO 80 II = 1, IP
                     WG(IP3+II) = ABMAT(II,LISTEND) - WG(II) 
   80             CONTINUE
                  SVAL(LISTEND) = ENORM2(WG, 4, IP, IP) 
C  
C                        ... COMPUTE THE VECTOR (N - C) AND ITS SQUARED NORM.
C
                  DO 90 II = 1, IP
                     WG(IP3+II) = EPSMAT(II,INDS+1) - WG(II) 
   90             CONTINUE
                  SVAL(MAXSIN+LISTEND) = ENORM2(WG, 4, IP, IP) 
C  
C                        ... COMPUTE THE VECTOR (N - W).
C
                  DO 100 II = 1, IP
                     WG(IP3+II) = EPSMAT(II,INDS+1) - ABMAT(II,LISTEND)
  100             CONTINUE
C  
C                        ... COMPUTE A PART OF LAMBDA.
C
                  IF (DABS(SVAL(LISTEND)) .LT. EPS .OR. 
     *               DABS(SVAL(MAXSIN+LISTEND)) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  SVAL(2*MAXSIN+LISTEND) = ENORM2(WG, 4, IP, IP)/
     *                              SVAL(LISTEND)/SVAL(MAXSIN+LISTEND)
C  
C                        ... END FOR THE COMPUTATIONS RELATED TO A!
C
                  IAC = .FALSE.
C  
C                        ... WE MUST CONSIDER ALSO B!
C
                  IBC = .TRUE.
C
C                        ... IF WE DON'T CONSIDER A ...
C
               ELSE IF (IBC) THEN
C
C                        ... BUT WE MUST STORE N IN B.
C
                  DO 110 II = 1, IP
                     ABMAT(II,LISTEND+MAXSIN) = EPSMAT(II,INDS)
  110             CONTINUE
C  
C                        ... END FOR THE COMPUTATIONS RELATED TO B!
C
                  IBC = .FALSE.
C
C                        ... IF WE DON'T CONSIDER A NOR B ...
C
               ELSE IF (INDS .NE. 0 .AND. INDS .EQ. (INDSAV(LISTBEG)+1))
     *                 THEN
C
C                        ... AND THE CURRENT INDEX INDS IS THE NEXT
C                            INDEX OF INSTABILITY IN INDSAV, WE PERFORM
C                            THE COMPUTATIONS RELATED TO D. 
C
                  LBM = LISTBEG + MAXSIN
                  LBM2 = LISTBEG + 2*MAXSIN
C  
C                        ... COMPUTE THE VECTOR (S - C) AND ITS SQUARED NORM.
C
                  DO 120 II = 1, IP
                     WG(IP3+II)  = WG(IP+II) - EPSMAT(II,INDS) 
  120             CONTINUE
                  TMP  = ENORM2(WG, 4, IP, IP) 
C  
C                        ... COMPUTE THE VECTOR (W - S).
C
                  DO 130 II = 1, IP
                     WG(IP3+II)  =  ABMAT(II,LISTBEG) - WG(IP+II) 
  130             CONTINUE
C  
C                        ... COMPUTE ANOTHER PART OF LAMBDA.
C
                  IF (DABS(SVAL(LISTBEG)) .LT. EPS .OR. 
     *               DABS(TMP) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  SVAL(LBM2) = SVAL(LBM2) + 
     *                  ENORM2(WG, 4, IP, IP)/SVAL(LISTBEG)/TMP
C  
C                        ... COMPUTE THE VECTOR (S - N).
C
                  DO 140 II = 1, IP
                     WG(IP3+II)  =  WG(IP+II) - ABMAT(II,LBM)  
  140             CONTINUE
C  
C                        ... END FOR COMPUTATION OF LAMBDA.
C
                  IF (DABS(SVAL(LBM)) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  SVAL(LBM2) = SVAL(LBM2) - 
     *                  ENORM2(WG, 4, IP, IP)/SVAL(LBM)/TMP
C  
C                        ... END FOR THE COMPUTATIONS RELATED TO D!
C
C  
C                        ... AND WE MUST COMPUTE E!
C
                  IEC = .TRUE.
               END IF
            END IF 
         ELSE
C
C                        ... WHEN THERE IS NO LOCAL INSTABILITY PRESENT
C                            NOR A NEW POINT OF INSTABILITY WE USE
C                            THE NORMAL RULE OF THE ALGORITHM.
C  
            RD = ENORM2(WG, 1, IP, IP)
            IF (DABS(RD) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            DO 150 II = 1, IP
               WG(II) = WG(II) / RD
               IF (INDS .NE. 0) WG(II) = WG(II) + EPSMAT(II,INDS-1)
  150       CONTINUE
         END IF
C  
C                        ... SHIFT THE RHOMBUS.
C
         DO 160 II = 1, IP
            IF (INDS .NE. 0) EPSMAT(II,INDS-1) = WG(IP2+II)
            WG(IP2+II) = WG(IP+II)
            WG(IP+II)   = WG(II)
  160    CONTINUE
C  
C                        ... END LOOP FOR INDS.
C
  170 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES OF THE DIAGONAL.
C
      DO 180 II = 1, IP
         EPSMAT(II,INDM-1) = WG(IP2+II)
         EPSMAT(II,INDM)   = WG(IP+II)
C  
C                        ... SET THE RESULT VALUE.
C
         IF (MOD(INDM,2) .EQ. 0) THEN
            EPSINI(II) = EPSMAT(II,INDM)
         ELSE
            EPSINI(II) = EPSMAT(II,INDM-1)
         END IF
  180 CONTINUE
      RETURN
      END
