C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - GAUSS                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           COMPUTES THE SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A AS    +
C           COEFFICIENT MATRIX AND N DIFFERENT RIGHT HAND SIDES.               +
C           THE METHOD IS GAUSSIAN ELIMINATION.                                +
C           THE COLUMNS N+1,....,2N OF A CONTAIN THE N RIGHT HAND SIDES AS     +
C           INPUT AND THE N SOLUTIONS AS OUTPUT.                               +
C                                                                              +
C   USAGE:                                                                     +
C           CALL GAUSS (A, N, IR, EPS, INDS)                                   +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT/OUTPUT REAL MATRIX CONTAINING THE COEFFICIENT       +
C                    MATRIX AND THE RIGHT HAND SIDES.                          +
C                                                                              +
C           N      - INPUT INTEGER EQUAL TO THE DIMENSION OF THE SYSTEM.       +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX A.                                             +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           INDS   - OUTPUT INTEGER VALUE.                                     +
C                    IF INDS = 0 THEN THE MATRIX A IS NON-SINGULAR.            +
C                    IF INDS = 1 THEN THE MATRIX A IS SINGULAR.                +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C        - IN THE CALLING PROCEDURE THE MINIMAL DIMENSION FOR THE MATRIX       +
C          ARGUMENT (IR=N) IS:                                                 +
C           A(N,N*2)                                                           +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE GAUSS (A, N, IR, EPS, INDS)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER INDS, IR, N
      DOUBLE PRECISION EPS
      DOUBLE PRECISION A(IR,1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER I, IPIVOT, J, K
      DOUBLE PRECISION TMP, PIVOT
C
C                        ... EXECUTABLE STATEMENTS
C
      INDS = 0
C
C                        ... TRIANGULARIZATION
C
      DO 50 K = 1, N-1
C
C                        ... SEARCH FOR THE PIVOT (PARTIAL PIVOTING)
C
         PIVOT = 0.0D0
         DO 10 I = K, N
            TMP = DABS(A(I,K))
            IF (TMP .GE. PIVOT) THEN
               IPIVOT = I
               PIVOT  = TMP 
            END IF
   10    CONTINUE
C
C                        ... TEST FOR SINGULAR MATRIX
C
         IF (DABS(PIVOT) .LT. EPS) THEN
            INDS = 1 
            RETURN
         END IF
C
C                        ... EXCHANGES THE ROW K WITH THE ROW 
C                            CONTAINING THE MAXIMUM PIVOT
C
         DO 20 J = K, N+N
            TMP         = A(K,J)
            A(K,J)      = A(IPIVOT,J)
            A(IPIVOT,J) = TMP 
   20    CONTINUE
C
C                        ... APPLIES THE ELEMENTS TRANSFORMATION
C
         DO 40 I = K+1, N
            IF (A(I,K) .NE. 0.0D0) THEN
               TMP = A(I,K) / A(K,K)
               DO 30 J = K+1, N+N
                  A(I,J) = A(I,J) - TMP * A(K,J)
   30          CONTINUE
            END IF
   40    CONTINUE
   50 CONTINUE
C
C                        ... TEST FOR SINGULAR MATRIX 
C                            (PIVOT OF THE LAST ROW)
C
      IF (DABS(A(N,N)) .LT. EPS) THEN
         INDS = 1 
         RETURN
      END IF
C
C                        ... SOLVING THE TRIANGULAR SYSTEMS
C                            FOR THE N RIGHT HAND SIDES 
C
      DO 80 K = 1, N
         A(N,N+K) = A(N,N+K) / A(N,N)
         DO 70 I = N-1, 1, -1
            DO 60 J = I+1, N
               A(I,N+K) = A(I,N+K) - A(I,J) * A(J,N+K)
   60       CONTINUE
            A(I,N+K) = A(I,N+K) / A(I,I)
   70    CONTINUE
   80 CONTINUE
      RETURN
      END
