C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - GTRAN                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SCALAR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE G-TRANSFORMATION.                      +
C                                                                              +
C   USAGE:                                                                     +
C           CALL GTRAN (GVEC, MAXCOL, INIT, EPS, GINIT, RSINIT,                +
C              RSVEC, IER)                                                     +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           GVEC   - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF GVEC MUST BE (0:IE), WITH IE GREATER         +
C                    OR EQUAL TO (2*MAXCOL-1). THIS VECTOR CONTAINS,           +
C                    AFTER THE k-TH CALL:                                      +
C                                                                              +
C                     - IF k = 1                                               +
C                                                                              +
C                        (0)                                                   +
C                       G                                                      +
C                        0                                                     +
C                                                                              +
C                     - IF 1 < k <= 2*MAXCOL+1 AND k ODD                       +
C                                                                              +
C                        (k-1)   (k-3)   (k-2)   (k-5)   (k-4)                 +
C                       G    ,  G    ,  G    ,  G   ,   G     ,...,            +
C                        0       1       1       2       2                     +
C                                 (3)         (0)        (1)                   +
C                                G       ,   G       ,  G                      +
C                                 (k-1)/2-1   (k-1)/2    (k-1)/2               +
C                                                                              +
C                     - IF 1 < k <  2*MAXCOL+1 AND k EVEN                      +
C                                                                              +
C                        (k-1)   (k-3)   (k-2)   (k-5)   (k-4)                 +
C                       G    ,  G    ,  G    ,  G   ,   G     ,...,            +
C                        0       1       1       2       2                     +
C                                 (1)         (2)        (0)                   +
C                                G       ,   G       ,  G                      +
C                                 k/2-1       k/2-1      k/2                   +
C                                                                              +
C                     - IF k > 2*MAXCOL+1                                      +
C                                                                              +
C                        (k-1)   (k-3)   (k-2)   (k-5)   (k-4)                 +
C                       G    ,  G    ,  G    ,  G   ,   G     ,...,            +
C                        0       1       1       2       2                     +
C                                  (k-2*MAXCOL-1)    (k-2*MAXCOL)              +
C                                 G             ,   G                          +
C                                  MAXCOL            MAXCOL                    +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN THAT    +
C                                                                   (j)        +
C                    THE USER WANTS TO COMPUTE (THE LOWER INDEX OF G   ).      +
C                                                                   m          +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           GINIT  - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    GINIT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE      +
C                                                                              +
C                     (0)                                                      +
C                    G            IF  k = 1                                    +
C                     0                                                        +
C                                                                              +
C                     (0)                                                      +
C                    G            IF  1 < k <  2*MAXCOL+1, k EVEN              +
C                     k/2                                                      +
C                                                                              +
C                     (1)                                                      +
C                    G            IF  1 < k <= 2*MAXCOL+1, k ODD               +
C                     (k-1)/2                                                  +
C                                                                              +
C                     (k-2*MAXCOL)                                             +
C                    G            IF  k > 2*MAXCOL+1                           +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)  (0)  (1)  (0)  (1)      (0)     (1)     (2)         +
C                    G ,  G  , G  , G  , G  ,..., G     , G     , G     , ...  +
C                     0    1    1    2    2        MAXCOL  MAXCOL  MAXCOL      +
C                                                                              +
C           RSINIT - INPUT REAL VALUE. IT MUST CONTAIN THE ELEMENT             +
C                    x   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.            +
C                     k-1                                                      +
C                                                                              +
C           RSVEC  - WORKING REAL VECTOR. IN THE CALLING PROCEDURE THE         +
C                    DIMENSION OF RSVEC MUST BE (0:IE), WITH IE GREATER        +
C                    OR EQUAL TO 2*(MAXCOL-1). THIS VECTOR CONTAINS THE        +
C                    r AND s VALUES.                                           +
C                                                                              +
C           IER    - INPUT/OUTPUT INDEX WARNING/ERROR.                         +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 400     WARNING. CANCELLATION ERROR. NUMERICAL      +
C                                  INSTABILITY PRESENT. THE USER MUST SET      +
C                                  AGAIN IER=0 IN THE CALLING PROGRAM TO       +
C                                  CONTINUE.                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION AND THE VECTORS GVEC AND RSVEC MUST NOT BE      +
C          MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE           +
C          SUBROUTINE.                                                         +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF G   (J <= MAXCOL) THE USER MUST CALL THIS    +
C                                  J                                           +
C          SUBROUTINE 2*J TIMES.                                               +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          VECTOR ARGUMENTS ARE:                                               +
C           GVEC(0:2*MAXCOL-1)                                                 +
C           RSVEC(0:2*(MAXCOL-1))                                              +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE GTRAN (GVEC, MAXCOL, INIT, EPS, GINIT, RSINIT,      
     *       RSVEC, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, MAXCOL
      DOUBLE PRECISION EPS, GINIT, RSINIT
      DOUBLE PRECISION GVEC(0:1), RSVEC(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER IND, INDM, INDS, INDSM1, MAXC2
      DOUBLE PRECISION AUX0, AUX1, AUX2, AUX3, TMP
      SAVE INDM, MAXC2
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER      = 0                                                           
         INIT     = 1
         INDM     = 0
         RSVEC(0) = RSINIT
         GVEC(0)  = GINIT
         MAXC2    = 2*MAXCOL - 1
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C
      IF (MAXCOL .EQ. 0) THEN
         RSVEC(0) = RSINIT
         GVEC(0)  = GINIT
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1).
C
      INDM = INDM + 1 
C
C                        ... IF INDM > 2*MAXCOL-1 THEN 
C                                         (NUMBER_OF_CALLS-2*MAXCOL)
C                            THE ELEMENT G 
C                                         MAXCOL
C                            IS COMPUTED.
C
      IF (INDM .LE. MAXC2) THEN
         IND = INDM
      ELSE 
         IND = MAXC2 
      END IF
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE 
C                            AUX1.
C
      AUX1   = RSINIT
C  
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS.
C
      DO 10 INDS = 0, IND - 1
C  
C                        ... COMPUTE THE VALUE OF THE ELEMENTS r OR s
C
         IF (DABS(RSVEC(INDS)) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         TMP  = AUX1 / RSVEC(INDS) - 1.0D0
         IF (DABS(TMP) .LT. EPS) IER = 400
         AUX0 = TMP
         IF (INDS .NE. 0) AUX0 = RSVEC(INDS-1) * AUX0
C  
C                        ... COMPUTE THE VALUE OF THE ELEMENTS G
C
         IF (MOD(INDS,2) .EQ. 0) THEN
            IF (DABS(TMP) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            INDSM1 = INDS - 1
            IF (INDS .EQ. 0) THEN
               AUX3    = GVEC(0) - (GINIT - GVEC(0)) / TMP
               GVEC(0) = GINIT
               IF (INDM .LE. 2) GINIT = AUX3
            ELSE
               GINIT = GVEC(INDSM1) - 
     *           (GVEC(INDS) - GVEC(INDSM1)) / TMP
C  
C                        ... SHIFT THE ELEMENTS OF THE G VECTOR.
C
               GVEC(INDSM1)  = GVEC(INDS)
               GVEC(INDS)    = AUX3
               AUX3          = GINIT
            END IF
         END IF
C  
C                        ... SHIFT THE ELEMENTS OF THE rs VECTOR.
C
         IF (INDS .NE. 0) RSVEC(INDS-1) = AUX2
         AUX2 = AUX1
         AUX1 = AUX0
C  
C                        ... END LOOP FOR INDS.
C
   10 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES.
C
      RSVEC(IND-1) = AUX2
      IF (IND .LT. MAXC2) RSVEC(IND) = AUX1
      GVEC(IND)    = GINIT
      RETURN
      END
