C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - HALGO                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION VIA THE H-ALGORITHM                  +
C                                                                              +
C   USAGE:                                                                     +
C           CALL HALGO (H, MAXCOL, INIT, EPS, HINIT, GINIT, IR, IP, G, IER)    +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           H      - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSIONS OF H MUST BE (IP,0:IE), WITH IE GREATER OR     +
C                    EQUAL TO MAXCOL. THIS MATRIX CONTAINS, AFTER THE k-TH     +
C                    CALL THE LAST COMPUTED ROW OF THE H-ARRAY:                +
C                                                                              +
C                                          (k-1)  (k-2)       (0)              +
C                     - IF k <= MAXCOL+1  H    , H     ,..., H                 +
C                                          0      1           k-1              +
C                                                                              +
C                                          (k-1)  (k-2)       (k-1-MAXCOL)     +
C                     - IF k > MAXCOL+1   H    , H     ,..., H                 +
C                                          0      1           MAXCOL           +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN         +
C                    OF THE H-ARRAY THAT THE USER WANTS TO COMPUTE             +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF H   ).                                +
C                                         k                                    +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           HINIT  - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VALUE       +
C                    OF S    BEFORE THE k-TH CALL OF THE SUBROUTINE.           +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                     (0)                                                      +
C                    H            IF  k <= MAXCOL+1                            +
C                     k-1                                                      +
C                                                                              +
C                     (k-1-MAXCOL)                                             +
C                    H            IF  k >  MAXCOL+1                            +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)   (0)   (0)         (0)      (1)      (2)            +
C                    H   , H   , H   , ... , H      , H      , H      , ...    +
C                     0     1     2           MAXCOL   MAXCOL   MAXCOL         +
C                                                                              +
C           GINIT  - INPUT REAL VECTOR. IN THE CALLING PROCEDURE THE           +
C                    DIMENSION OF GINIT MUST BE GREATER OR EQUAL TO 2*MAXCOL.  +
C                    BEFORE THE k-TH CALL OF THE SUBROUTINE IT MUST CONTAIN    +
C                    IF k = 1   GINIT(1)= G (0)                                +
C                                          1                                   +
C                                                                              +
C                    IF k = 2   GINIT(1)= G (1)                                +
C                                          1                                   +
C                                                                              +
C                    IF 3 <= k <= MAXCOL+1                                     +
C                                                                              +
C                       GINIT=[G (k-1), G (k-1), ... , G   (k-1),              +
C                               1        2              k-2                    +
C                                                                              +
C                              G   (0), G   (1), ... , G   (k-1)]              +
C                               k-1      k-1            k-1                    +
C                                                                              +
C                    IF k > MAXCOL+1                                           +
C                                                                              +
C                       GINIT=[G (k-1), G (k-1), ... , G   (k-1)]              +
C                               1        2              MAXCOL                 +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRIX H, AND DIMENSION OF THE VECTOR HINIT.              +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF ROWS FOR   +
C                    THE MATRIX H AND DIMENSION OF THE VECTOR HINIT)           +
C                                                                              +
C           G      - WORKING REAL VECTOR IN WHICH THE LAST ROW AND THE         +
C                    LAST COLUMN OF EACH G  ARRAY ARE SAVED FOR THE NEXT       +
C                                         i                                    +
C                    CALL OF THE SUBROUTINE. IN THE CALLING PROCEDURE THE      +
C                    DIMENSION OF G MUST BE GREATER OR EQUAL TO                +
C                    MAXCOL*(MAXCOL+1) AND TO MAXCOL*NUMBER_OF_CALLS.          +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO THE VECTOR SPACE      +
C                                  DIMENSION.                                  +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE MATRIX H AND THE VECTOR G MUST NOT BE      +
C          MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE           +
C          SUBROUTINE.                                                         +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF H   (J <= MAXCOL), THE USER MUST CALL THIS   +
C                                  J                                           +
C          SUBROUTINE (J+1) TIMES.                                             +
C       -  IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRIX H,      +
C          AND THE DIMENSIONS OF THE VECTOR HINIT MUST BE THE SAME.            +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=IP) ARE:                            +
C           H(IP,0:MAXCOL)                                                     +
C           HINIT(IP)                                                          +
C           GINIT(2*MAXCOL)                                                    +
C           G(MAXDIM),MAXDIM=MAX(MAXCOL*(MAXCOL+1),MAXCOL*NUMBER_OF_CALLS)     +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE HALGO (H, MAXCOL, INIT, EPS, HINIT, GINIT, IR, 
     *       IP, G, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IP, IR, MAXCOL
      DOUBLE PRECISION EPS
      DOUBLE PRECISION H(IR,0:1), HINIT(1), G(1), GINIT(1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      DOUBLE PRECISION R, RD
      INTEGER I, II, IND, INDI, INDI1, INDJ, INDM, J
      SAVE INDM
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IF (IP .GT. IR) THEN
            IER = 300
            RETURN
         END IF
         IER    = 0                                                           
         INDM   = 0
         INIT   = 1
         G(1)   = GINIT(1)
         DO 10 II = 1, IP
            H(II,0) = HINIT(II)
   10    CONTINUE
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         DO 20 II = 1, IP
            H(II,0) = HINIT(II)
   20    CONTINUE
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS -1)
C
      INDM = INDM+1
C
C                        ... ADD AN ELEMENT TO G  ARRAY AT EACH CALL
C                                               1
C
      G(1+INDM*MAXCOL) = GINIT(1)
      IF (INDM .NE. 1) THEN
C
C                        ... COMPUTE AND SAVE IN G THE NECESSARY ELEMENTS 
C                            OF THE G ARRAYS (ONLY FROM THE THIRD CALL)
C                                    i
C
         IF (INDM .GE. 3) THEN
C
C                        ... COMPUTE THE NEXT ROW OF ALL THE PREVIOUS G  
C                                                                      i
C                            ARRAYS (i = 2 TO INDM-1 IF INDM <= MAXCOL+1)    
C                                   (i = 2 TO MAXCOL IF INDM > MAXCOL+1)
C                            AND SAVE IN G THE NEW COMPUTED ELEMENTS 
C
            IND = MIN0(INDM-1,MAXCOL)
            DO 40 I = 2, IND
               DO 30 J = 1, I-1
                  INDI = I + (J-1) * MAXCOL
                  INDJ = J + INDM * MAXCOL
                  R    = G(INDJ-MAXCOL) - G(INDJ) 
                  IF (DABS(R) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  R  = GINIT(I) - (G(INDI)-GINIT(I))/R * G(INDJ)
                  G(INDI)  = GINIT(I)
                  GINIT(I) = R
   30          CONTINUE
               G(I+INDM*MAXCOL) = R
   40       CONTINUE
         END IF
         IF (INDM .LE. MAXCOL) THEN
C
C                        ... COMPUTE THE NEW G     ARRAY  IF INDM <= MAXCOL
C                                             INDM
C                            AND SAVE IN G THE RELATIVE LAST ROW AND COLUMN
C
            G(INDM) = GINIT(INDM)
            DO 60 I = 1, INDM
               INDI1 = INDM + I
               IND   = MIN0(I,INDM-1)
               DO 50 J = 1, IND
                  INDI = J + I*MAXCOL
                  INDJ = INDM + (J-1)*MAXCOL
                  R    = G(INDI-MAXCOL) - G(INDI)
                  IF (DABS(R) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  R  = GINIT(INDI1) - (G(INDJ)-GINIT(INDI1))/R * G(INDI)
                  G(INDJ)      = GINIT(INDI1)
                  GINIT(INDI1) = R
   50          CONTINUE
               G(INDM+I*MAXCOL) = R
   60       CONTINUE
         END IF
      END IF
C
C                        ... COMPUTE THE NEW ELEMENTS OF THE NEXT ROW
C                            IN THE H-ARRAY
C
      IND = MIN0(INDM,MAXCOL)
      DO 80 I = 1, IND
         INDI  = I + INDM * MAXCOL
         RD    = G(INDI-MAXCOL) - G(INDI)
         IF (DABS(RD) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         RD = G(INDI) / RD
         DO 70 II = 1, IP
            R         = HINIT(II) - (H(II,I-1) - HINIT(II)) * RD
            H(II,I-1) = HINIT(II)
            HINIT(II) = R
            IF (I .EQ. IND) H(II,IND) = R
   70    CONTINUE
C
C                        ... END LOOP FOR I.
C
   80 CONTINUE
      RETURN
      END
