C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   FUNCTION NAME       - IASSER                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           LOGICAL FUNCTION WHICH CHECKS THE ASSERTION :                      +
C           INTERSECTION FOR J = K,...,N OF THE INTERVALS WITH BOUNDS SN(J)    +
C           AND TCJ NOT EMPTY, WHERE TCJ = C * TN(J) + (1 - C) * SN(J).        +
C           IASSER = .TRUE. IF THE INTERSECTION IS NOT EMPTY.                  +
C           IASSER = .FALSE. IF THE INTERSECTION IS EMPTY.                     +
C           <A1, B1> IS THE INTERSECTION OF THE PREVIOUS INTERVALS.            +
C           <A2, B2> IS THE CURRENT INTERVAL.                                  +
C           <A, B> = [SN(J), TCJ] IF SN(J) <= TCJ                              +
C           <A, B> = [TCJ, SN(J)] IF SN(J) > TCJ                               +
C                                                                              +
C   USAGE:                                                                     +
C           IASSER (SN, TN, C, K, N)                                           +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           SN     - INPUT REAL VECTOR OF DIMENSION (-1:M). IN THE             +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF SN         +
C                    MUST BE GREATER OR EQUAL TO N.                            +
C                    THIS VECTOR CONTAINS THE VALUES OF THE SEQUENCE S .       +
C                                                                     n        +
C                                                                              +
C           TN     - INPUT REAL VECTOR OF DIMENSION (-1:M). IN THE             +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF TN         +
C                    MUST BE GREATER OR EQUAL TO N.                            +
C                    THIS VECTOR CONTAINS THE VALUES OF THE SEQUENCE T .       +
C                                                                     n        +
C                                                                              +
C           C      - INPUT REAL GIVING THE CONSTANT TO BE USED IN THE          +
C                    COMPUTATION OF T (n) = c * T  + (1 - c) * S .             +
C                                    c           n              n              +
C                                                                              +
C           K      - INPUT INTEGER, LOWER INDEX FOR THE BOUNDS.                +
C                                                                              +
C           N      - INPUT INTEGER, UPPER INDEX FOR THE BOUNDS.                +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION.                                             +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENTS ARE:                                                      +
C           SN(-1:N)                                                           +
C           TN(-1:N)                                                           +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      LOGICAL FUNCTION IASSER (SN, TN, C, K, N)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER K, N
      DOUBLE PRECISION C
      DOUBLE PRECISION SN(-1:1), TN(-1:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      INTEGER J
      DOUBLE PRECISION A1, B1, A2, B2, TMP
C
C                        ... EXECUTABLE STATEMENTS
C
      IASSER = .TRUE.
      IF (N .EQ. K) THEN
C
C                        ... LOWER BOUND EQUAL TO UPPER BOUND:
C                            IASSER ALWAYS TRUE
C
         RETURN
      END IF
C
C                        ... COMPUTE A1-B1 STARTING INTERVAL
C
      A1 = SN(K)
      B1 = C * TN(K) + (1.0D0 - C) * SN(K)
      IF (A1 .GT. B1) THEN
C
C                        ... EXCHANGE B1 WITH A1 
C
         TMP  = A1
         A1   = B1
         B1   = TMP
      END IF
C
C                        ... STARTING LOOP 
C
      DO 10 J = K+1, N
         A2 = SN(J)
         B2 = C * TN(J) + (1.0D0 - C) * SN(J)
         IF (A2 .GT. B2) THEN
C
C                        ... EXCHANGE B2 WITH A2 
C
            TMP = A2
            A2  = B2
            B2  = TMP
         END IF
         IF (B1 .LT. A2 .OR. B2 .LT. A1) THEN
C
C                        ... THE INTERVALS ARE : 
C                        ... A1___B1   A2...B2   OR   A2...B2   A1___B1
C
            IASSER = .FALSE.
            RETURN
         END IF
C
C                        ... RESEARCH OF THE NEW INTERSECTION
C
         IF (A1 .LT. A2) A1 = A2
C
C                        ... THE INTERVALS ARE : 
C                        ... A1___A2___B1   B2   OR   A1___A2...B2___B1
C
         IF (B1 .GE. B2) B1 = B2
C
C                        ... THE INTERVALS ARE : 
C                        ... A2   A1___B2___B1   OR   A1___A2...B2___B1
C
C                        ... OTHERWISE
C
C                        ... THE INTERVALS ARE : A2...A1___B1...B2
C
   10 CONTINUE
      RETURN
      END
