C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - INVLAP                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           LAPLACE TRANSFORM INVERSION.                                       +
C           LET THE FUNCTION F(p) BE KNOWN IN (2k+2) DISTINCT POINTS.          +
C           THE SUBROUTINE INVERTS AN INTERPOLATING RATIONAL FUNCTION WITH A   +
C           NUMERATOR OF DEGREE k AND A DENOMINATOR OF DEGREE (k+1) IN THE     +
C           UNKNOWN p.                                                         +
C           THE RATIONAL FUNCTION IS OBTAINED BY THE ALGORITHM FOR RATIONAL    +
C           INTERPOLATION THAT USES THE RHO-ALGORITHM AND CONTINUED FRACTIONS. +
C           THE INVERSION IS PERFORMED WITHOUT DECOMPOSING THE RATIONAL        +
C           FRACTION INTO PARTIAL FRACTIONS.                                   +
C                                                                              +
C   USAGE:                                                                     +
C           CALL INVLAP (KDEG, P, F, EPS, NV, V, WK, IER)                      +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           KDEG   - INPUT INTEGER. IT REPRESENTS THE DEGREE k OF THE          +
C                    NUMERATOR OF THE INTERPOLATING RATIONAL FUNCTION.         +
C                    ITS VALUE MUST BE GREATER OR EQUAL TO 1.                  +
C                                                                              +
C           P      - INPUT REAL VECTOR. IT MUST CONTAINS THE (2k+2) DISTINCT   +
C                    INTERPOLATION POINTS p , n = 0,...,2k+1. IN THE CALLING   +
C                                          n                                   +
C                    PROCEDURE THE DIMENSION OF P MUST BE (0:IP) WITH IP       +
C                    GREATER OR EQUAL TO 2k+1.                                 +
C                                                                              +
C           F      - INPUT REAL VECTOR. IT MUST CONTAINS THE VALUES OF THE     +
C                    FUNCTION F(p) AT THE POINTS p  . ALL THESE VALUES MUST BE +
C                    DIFFERENT FROM ZERO.         n                            +
C                    IN THE CALLING PROCEDURE THE DIMENSION OF F MUST BE       +
C                    (0:IP) WITH IP GREATER OR EQUAL TO 2k+1.                  +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           NV     - INPUT INTEGER VALUE. IT REPRESENTS THE INDEX OF THE LAST  +
C                    COEFFICIENT v  OF THE SERIES GIVING f, THAT THE USER      +
C                                 I                                            +
C                    WANTS TO COMPUTE.                                         +
C                                                                              +
C                                                                              +
C           V      - OUTPUT REAL VECTOR. IT CONTAINS THE VALUES v  (I=0,..,NV) +
C                                                                I             +
C                    OF THE COEFFICIENTS OF THE SERIES GIVING f(t).            +
C                    THE COEFFICIENTS HAVE BEEN MULTIPLIED BY THE CONSTANT A.  +
C                    THE SERIES CAN BE COMPUTED AS THE SUM (FROM I=0 TO NV)    +
C                    OF V(I)* t**I / I!.                                       +
C                    IN THE CALLING PROCEDURE THE DIMENSION OF V MUST BE       +
C                    (0:IV) WITH IV GREATER OR EQUAL TO NV.                    +
C                                                                              +
C           WK     - WORKING REAL VECTOR OF DIMENSION (0:M) WITH M GREATER OR  +
C                    EQUAL TO MAX0(8*KDEG+13,3*KDEG+NV+2).                     +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER =  100    DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 1500    AT LEAST TWO COMPONENTS OF THE VECTOR P ARE +
C                                  EQUAL.                                      +
C                    IER = 1600    AT LEAST ONE COMPONENT OF THE VECTOR F IS   +
C                                  ZERO.                                       +
C                    IER = 1700    ALL THE COEFFICIENTS OF THE NUMERATOR OR OF +
C                                  THE DENOMINATOR ARE ZERO.                   +
C                    IER = 1800    THE DEGREE OF THE DENOMINATOR IS NOT        +
C                                  GREATER THAN THAT OF THE NUMERATOR.         +
C                                  THE METHOD CANNOT BE USED.                  +
C                    IER = 1900    THE DEGREE k IS LESS THAN 1.                +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE IN +
C          DOUBLE PRECISION.                                                   +
C       -  BETTER RESULTS COULD BE OBTAINED BY USING THE SUBROUTINE EPSRHO     +
C          FOR THE RHO-ALGORITHM.                                              +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENTS ARE:                                                      +
C           P(0:2*KDEG+1)                                                      +
C           F(0:2*KDEG+1)                                                      +
C           V(0:NV)                                                            +
C           WK(0:MAX0(8*KDEG+13,3*KDEG+NV+2))                                  +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE INVLAP (KDEG, P, F, EPS, NV, V, WK, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER KDEG, NV, IER
      DOUBLE PRECISION EPS 
      DOUBLE PRECISION P(0:1), F(0:1), V(0:1), WK(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C                            
      INTEGER I, J, KM, NPT, NPT1, NPT2, NPT3, NPT4, NPT5, NPT6, 
     *        NDEG, MDEG
      DOUBLE PRECISION ACOEF
      LOGICAL*1 EVEN, NFIND
C
C                        ... EXECUTABLE STATEMENTS
C
      IER  = 0
      EVEN = .FALSE.
C
C                        ... CONTROLS THE VALUE OF KDEG
C
      IF (KDEG .LT. 1) THEN
         IER = 1900
         RETURN
      END IF
C
      NPT  = 2 * KDEG + 1
      NPT1 = NPT + 1
      NPT2 = 2 * NPT1
      NPT3 = 3 * NPT1
C
C                        ... CONTROLS THAT THE POINTS P ARE DISTINCT
C
      DO 20 J = 0, NPT-1
         DO 10 I = J+1, NPT
            IF (P(J) .EQ. P(I)) THEN
               IER = 1500
               RETURN
            END IF
   10    CONTINUE
   20 CONTINUE
C
      DO 30 I = 0, NPT
C
C                        ... CONTROLS THAT THE VALUES F ARE NOT ZERO
C
         IF (F(I) .EQ. 0.0D0) THEN
            IER = 1600
            RETURN
         END IF
C
C                        ... INITIALIZATIONS FOR THE RHO-ALGORITHM
C
         WK(I)      = 0.0D0
         WK(NPT1+I) = 1.0D0 / F(I)
   30 CONTINUE
      WK(NPT3) = WK(NPT1)
C
C                        ... COMPUTES THE NEXT COLUMNS OF THE 
C                            RHO-ALGORITHM
C
      DO 60 I = 1, NPT
         DO 40 J = 0, NPT-I
            KM    = NPT1 + J 
            ACOEF = WK(KM+1) - WK(KM)
            IF (DABS(ACOEF) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            WK(NPT2+J) = WK(J+1) + (P(J+I) - P(J)) / ACOEF
   40    CONTINUE
         DO 50 J = 0, NPT-I
            KM     = NPT1 + J   
            WK(J)  = WK(KM)
            WK(KM) = WK(NPT2+J)
   50    CONTINUE
         WK(NPT1-I) = WK(NPT2-I)
         WK(NPT3+I) = WK(NPT1)
   60 CONTINUE
C
C                        ... RECURSIVE COMPUTATION OF THE INTERPOLATING 
C                            RATIONAL FUNCTION VIA THE CONVERGENTS OF
C                            THE CONTINUED FRACTION
C
      WK(0) = WK(NPT3)
      WK(1) = WK(NPT3+1) 
      DO 70 I = 2, NPT
         KM    = NPT3 + I
         WK(I) = WK(KM) - WK(KM-2)
   70 CONTINUE
C
      NPT2 = 3 * KDEG + 4
      NPT3 = 4 * KDEG + 6
      NPT4 = 5 * KDEG + 8
      NPT5 = 6 * KDEG + 10
      NPT6 = 7 * KDEG + 12
C
C                        ... FIRST CONVERGENTS
C
      WK(NPT1) = 1.0D0
      WK(NPT2) = WK(0)
      WK(NPT4) = 0.0D0
      WK(NPT5) = 1.0D0
C
C                        ... NEXT CONVERGENTS
C
      DO 100 I = 1, NPT
C
C                        ... FIRST COEFFICIENTS
C
         WK(NPT3) = WK(I) * WK(NPT2) - P(I-1) * WK(NPT1)
         WK(NPT6) = WK(I) * WK(NPT5) - P(I-1) * WK(NPT4)
C
C                        ... NEXT COEFFICIENTS
C
         EVEN = MOD(I,2) .EQ. 0
         IF (EVEN) THEN
            KM = I / 2
         ELSE
            KM = (I+1) / 2
         END IF
         IF (I .GE. 3) THEN
            DO 80 J = 1, KM-1
               NDEG       = NPT1 + J
               MDEG       = NPT4 + J
               WK(NPT3+J) = WK(I) * WK(NPT2+J) - P(I-1) 
     *                            * WK(NDEG) + WK(NDEG-1)
               IF (EVEN .OR. J .NE. KM-1) 
     *            WK(NPT6+J) = WK(I) * WK(NPT5+J) - P(I-1) 
     *                               * WK(MDEG) + WK(MDEG-1)
   80       CONTINUE
         END IF
C
C                        ... LAST COEFFICIENTS
C
         IF (EVEN) THEN
            WK(NPT3+KM) = WK(I) * WK(NPT2+KM) + WK(NPT1+KM-1)
            WK(NPT6+KM) = WK(NPT4+KM-1)
         ELSE
            WK(NPT3+KM) = WK(NPT1+KM-1)
            IF (I .NE. 1) 
     *         WK(NPT6+KM-1) = WK(I) * WK(NPT5+KM-1) + WK(NPT4+KM-2)
         END IF
         DO 90 J = 0, KM
            NDEG       = NPT2 + J
            MDEG       = NPT5 + J
            WK(NPT1+J) = WK(NDEG)
            WK(NDEG)   = WK(NPT3+J)
            WK(NPT4+J) = WK(MDEG)
            WK(MDEG)   = WK(NPT6+J)
   90    CONTINUE
  100 CONTINUE
C
C                        ... END OF THE COMPUTATION OF THE
C                            INTERPOLATING RATIONAL FUNCTION
C
C
C                        ... COMPUTES THE DEGREES OF THE NUMERATOR AND
C                            OF THE DENOMINATOR OF THE RATIONAL FUNCTION
C
      NFIND = .TRUE.
      DO 110 MDEG = KDEG, 1, -1
         IF (DABS(WK(NPT6+MDEG)) .GT. EPS) GO TO 120
  110 CONTINUE
      NFIND = .FALSE.
  120 CONTINUE
      IF (NFIND) THEN
         DO 130 NDEG = KDEG+1, 1, -1
            IF (DABS(WK(NPT3+NDEG)) .GT. EPS) GO TO 140
  130    CONTINUE
         NFIND = .FALSE.
  140    CONTINUE  
      END IF
      IF (.NOT. NFIND) THEN
         IER = 1700
         RETURN
      END IF
C
C                        ... CONTROL ON THE DEGREES
C
      IF (MDEG .GE. NDEG) THEN
         IER = 1800
         RETURN
      END IF
C
C                        ... COMPUTATION OF ACOEF
C
      ACOEF = WK(NPT6+MDEG) / WK(NPT3+NDEG)
C
C                        ... NORMALIZATION OF THE NUMERATOR
C
      DO 150 I = 1, MDEG
         KM    = NPT6 + MDEG
         WK(I) = WK(KM-I) / WK(KM)
  150 CONTINUE
C
C                        ... NORMALIZATION OF THE DENOMINATOR
C
      DO 160 I = 1, NDEG
         KM         = NPT3 + NDEG
         WK(KDEG+I) = WK(KM-I) / WK(KM)
  160 CONTINUE
C
C                        ... COMPUTATION OF THE U'S AND THE V'S
C
C
C                        ... INITIALIZATIONS FOR THE U'S 
C
      IF (NDEG .GE. 2) THEN
         DO 170 I = 0, NDEG-2
            WK(NPT1+I) = 0.0D0
  170    CONTINUE
      END IF
      WK(NPT1+NDEG-1) = 1.0D0
C
C                        ... COMPUTATION OF THE V'S 
C
      DO 200 I = 0, NV
         KM   = NPT1 + I + MDEG
         V(I) = WK(KM)
         DO 180 J = 1, MDEG
            V(I) = V(I) + WK(J) * WK(KM-J)
  180    CONTINUE
         V(I) = ACOEF * V(I)
         IF (I .NE. NV) THEN
C
C                        ... COMPUTATION OF THE NEXT U
C
            KM     = NPT1 + I + NDEG
            WK(KM) = 0.0D0
            DO 190 J = 1, NDEG
               WK(KM) = WK(KM) - WK(KDEG+J) * WK(KM-J)
  190       CONTINUE
         END IF
  200 CONTINUE
C
      RETURN
      END
