C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - OVERHO                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SCALAR SEQUENCE EXTRAPOLATION VIA OVERHOLT-PROCESS                 +
C                                                                              +
C   USAGE:                                                                     +
C           CALL OVERHO (V, MAXCOL, INIT, EPS, VINIT, IER)                     +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           V      - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF V MUST BE (0:IE), WITH IE GREATER OR EQUAL   +
C                    TO MAXCOL. THIS VECTOR CONTAINS, AFTER THE k-TH CALL THE  +
C                    LAST COMPUTED ROW OF THE V-ARRAY:                         +
C                                                                              +
C                                              (0)                             +
C                     - IF k = 1 or k = 2     V                                +
C                                              0                               +
C                                                                              +
C                                              (k-2)  (k-3)       (0)          +
C                     - IF 2 < k <= MAXCOL+2  V    , V     ,..., V             +
C                                              0      1           k-2          +
C                                                                              +
C                                              (k-2)  (k-3)       (k-2-MAXCOL) +
C                     - IF k > MAXCOL+2       V    , V     ,..., V             +
C                                              0      1           MAXCOL       +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN         +
C                    OF THE V-ARRAY THAT THE USER WANTS TO COMPUTE             +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF V   ).                                +
C                                         k                                    +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           VINIT  - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S    BEFORE THE k-TH CALL OF THE SUBROUTINE.           +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                     (0)                                                      +
C                    V            IF  k = 1 OR k = 2                           +
C                     0                                                        +
C                                                                              +
C                     (0)                                                      +
C                    V            IF  2 < k <= MAXCOL+2                        +
C                     k-2                                                      +
C                                                                              +
C                     (k-2-MAXCOL)                                             +
C                    V            IF  k >  MAXCOL+2                            +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)  (0)  (0)  (0)       (0)      (1)      (2)           +
C                    V  , V  , V  , V  ,... , V      , V      , V      , ...   +
C                     0    0    1    2         MAXCOL   MAXCOL   MAXCOL        +
C                                                                              +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION AND THE VECTOR V MUST NOT BE MODIFIED BY     +
C          THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.           +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF V   (J <= MAXCOL) THE USER MUST CALL THIS    +
C                                  J                                           +
C          SUBROUTINE (J+2) TIMES.                                             +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSION FOR THE VECTOR       +
C          ARGUMENT IS:                                                        +
C           V(0:MAXCOL)                                                        +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE OVERHO (V, MAXCOL, INIT, EPS, VINIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, MAXCOL
      DOUBLE PRECISION VINIT, EPS
      DOUBLE PRECISION V(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      DOUBLE PRECISION DSE, DSO, TMP, VPREC
      INTEGER I, INDM, IND
      SAVE INDM, DSE, DSO, VPREC
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER    = 0
         INDM   = 0
         INIT   = 1
         V(0)   = VINIT
         VPREC  = VINIT
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C
      IF (MAXCOL .EQ. 0) THEN
         V(0) = VINIT
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE NUMBER_OF_CALLS - 1
C
      INDM = INDM + 1
C
C                        ... COMPUTE THE VALUE OF DELTA S 
C                                                        INDM-1
C
      IF (MOD(INDM,2) .EQ. 0) THEN
         DSE = VINIT - VPREC
      ELSE
         DSO = VINIT - VPREC
      END IF
      IF (INDM .EQ. 1) THEN
         VPREC  = VINIT
         VINIT  = V(0)
         RETURN
      END IF
C
C                                                  (INDM-1-I)
C                        ... COMPUTE THE ELEMENTS V        WITH
C                                                  I
C                              I = 1, 2, ..., INDM-1 IF INDM <= MAXCOL+1
C                              I = 1, 2, ..., MAXCOL IF INDM >  MAXCOL+1
C
      IND = MIN0(INDM-1,MAXCOL)
      DO 10 I = 1, IND
         TMP  = DSE**I - DSO**I
         IF (DABS(TMP) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         IF (MOD(INDM,2) .EQ. 0) THEN
            TMP  = V(I-1) - (VPREC - V(I-1))/TMP * DSO**I
         ELSE
            TMP  = V(I-1) + (VPREC - V(I-1))/TMP * DSE**I
         END IF
         V(I-1) = VPREC
         VPREC  = TMP
   10 CONTINUE
C
C                        ... SAVE FOR THE FINAL VALUES.
C
      VPREC  = VINIT
      V(IND) = TMP
C
C                        ... SET THE RESULT VALUE.
C
      VINIT  = TMP
      RETURN
      END
