C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - RICHAR                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SCALAR SEQUENCE EXTRAPOLATION VIA RICHARDSON PROCESS               +
C                                                                              +
C   USAGE:                                                                     +
C           CALL RICHAR (T, MAXCOL, INIT, EPS, TINIT, XAUX, XINIT, IER)        +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           T      - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF T MUST BE (0:IE), WITH IE GREATER OR EQUAL   +
C                    TO MAXCOL. THIS VECTOR CONTAINS, AFTER THE k-TH CALL THE  +
C                    LAST COMPUTED ROW OF THE T-ARRAY:                         +
C                                                                              +
C                                          (k-1)  (k-2)       (0)              +
C                     - IF k <= MAXCOL+1  T    , T     ,..., T                 +
C                                          0      1           k-1              +
C                                                                              +
C                                          (k-1)  (k-2)       (k-1-MAXCOL)     +
C                     - IF k > MAXCOL+1   T    , T     ,..., T                 +
C                                          0      1           MAXCOL           +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN         +
C                    OF THE E-ARRAY THAT THE USER WANTS TO COMPUTE             +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF T   ).                                +
C                                         k                                    +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           TINIT  - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S    BEFORE THE k-TH CALL OF THE SUBROUTINE.           +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                     (0)                                                      +
C                    T            IF  k <= MAXCOL+1                            +
C                     k-1                                                      +
C                                                                              +
C                     (k-1-MAXCOL)                                             +
C                    T            IF  k >  MAXCOL+1                            +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)   (0)   (0)         (0)      (1)      (2)            +
C                    T   , T   , T   , ... , T      , T      , T      , ...    +
C                     0     1     2           MAXCOL   MAXCOL   MAXCOL         +
C                                                                              +
C                                                                              +
C           XAUX   - WORKING REAL VECTOR TO STORE THE VALUES OF XINIT GIVEN    +
C                    AT EACH CALL BY THE USER. IN THE CALLING PROCEDURE THE    +
C                    DIMENSION OF XAUX MUST BE THE SAME THAN THAT OF T.        +
C                                                                              +
C           XINIT  - INPUT REAL VALUE. IT MUST CONTAIN THE ELEMENT x    OF     +
C                                                                   k-1        +
C                    THE AUXILIARY SEQUENCE, BEFORE THE  k-TH CALL OF          +
C                    THE SUBROUTINE.                                           +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION AND THE VECTORS T AND XAUX MUST NOT BE       +
C          MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE           +
C          SUBROUTINE.                                                         +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF T   (J <= MAXCOL) THE USER MUST CALL THIS    +
C                                  J                                           +
C          SUBROUTINE (J+1) TIMES.                                             +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENTS ARE:                                                      +
C           T(0:MAXCOL)                                                        +
C           XAUX(0:MAXCOL)                                                     +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE RICHAR (T, MAXCOL, INIT, EPS, TINIT, XAUX, XINIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, MAXCOL
      DOUBLE PRECISION EPS, TINIT, XINIT
      DOUBLE PRECISION T(0:1), XAUX(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      DOUBLE PRECISION TMP
      INTEGER I, IND, INDM, INDXL, INDXU, MAXCP1
      SAVE INDM, MAXCP1
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER     = 0
         INDM    = 0
         INIT    = 1
         MAXCP1  = MAXCOL+1
         T(0)    = TINIT
         XAUX(0) = XINIT
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         T(0)    = TINIT
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE NUMBER_OF_CALLS - 1
C
      INDM = INDM + 1
C
C                        ... IF INDM .GT. MAXCOL THEN THE VALUE
C                             (NUMBER_OF_CALLS-MAXCOL-1)
C                            T                          IS COMPUTED.
C                             (MAXCOL)
C
      IND = MIN0(INDM,MAXCOL)
C
C                        ... STORE THE XINIT VALUE.
C
      INDXL       = MOD(INDM,MAXCP1)
      XAUX(INDXL) = XINIT
C
C                        ... COMPUTE THE ELEMENTS OF THE NEXT ROW
C                            IN THE T-ARRAY
C

      DO 10 I = 1, IND
         INDXU = MOD(INDM-I,MAXCP1)
         TMP   = XAUX(INDXL) - XAUX(INDXU) 
         IF (DABS(TMP) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         TMP    = T(I-1) - (TINIT - T(I-1))/TMP * XAUX(INDXU)
         T(I-1) = TINIT
         TINIT  = TMP
   10 CONTINUE
C
C                        ... SAVE FOR THE FINAL VALUE.
C
      T(IND) = TMP
      RETURN
      END
