C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - RSALGO                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SCALAR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE rs-ALGORITHM.                          +
C                                                                              +
C   USAGE:                                                                     +
C           CALL RSALGO (RSVEC, MAXCOL, INIT, EPS, RSINIT, IFLAG, IER)         +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           RSVEC  - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF RSVEC MUST BE (0:IE), WITH IE GREATER        +
C                    OR EQUAL TO 2*(MAXCOL-1) IF IFLAG = 0, AND TO             +
C                    (2*MAXCOL-1) IF IFLAG = 1. THIS VECTOR CONTAINS, AFTER    +
C                    THE k-TH CALL:                                            +
C                                                                              +
C                     - IF k <= 2*MAXCOL-1 AND k ODD                           +
C                                                                              +
C                        (k-1)  (k-2)  (k-3)  (k-4)       (0)                  +
C                       r    , s    , r    , s    , ..., r                     +
C                        1      1      2      2           (k-1)/2+1            +
C                                                                              +
C                                                                              +
C                     - IF k < 2*MAXCOL-1 AND k EVEN                           +
C                                                                              +
C                        (k-1)  (k-2)  (k-3)  (k-4)       (0)                  +
C                       r    , s    , r    , s    , ..., s                     +
C                        1      1      2      2           k/2                  +
C                                                                              +
C                     - IF k > 2*MAXCOL-1 AND IFLAG = 0                        +
C                                                                              +
C                        (k-1) (k-2) (k-3) (k-4)      (0)       (k-2*MAXCOL+1) +
C                       r   , s   , r   , s   , ..., r   ,..., r               +
C                        1     1     2     2          MAXCOL    MAXCOL         +
C                                                                              +
C                     - IF k > 2*MAXCOL-1 AND IFLAG = 1                        +
C                                                                              +
C                        (k-1) (k-2) (k-3) (k-4)      (0)         (k-2*MAXCOL) +
C                       r   , s   , r   , s   , ..., s    , ..., s             +
C                        1     1     2     2          MAXCOL      MAXCOL       +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN THAT    +
C                                                                   (j)    (j) +
C                    THE USER WANTS TO COMPUTE (THE LOWER INDEX OF r   OR s   )+
C                                                                   m      m   +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           RSINIT - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                        (k-1)                                                 +
C                    OF r     = x   , BEFORE THE  k-TH CALL OF THE             +
C                        1       k-1                                           +
C                    SUBROUTINE. RSINIT CONTAINS AFTER THE k-TH CALL OF THE    +
C                    SUBROUTINE                                                +
C                                                                              +
C                     (0)                                                      +
C                    r            IF  k <= 2*MAXCOL-1, k ODD                   +
C                     (k-1)/2+1                                                +
C                                                                              +
C                     (0)                                                      +
C                    s            IF  k <= 2*MAXCOL,   k EVEN                  +
C                     k/2                                                      +
C                                                                              +
C                     (k-2*MAXCOL+1)                                           +
C                    r            IF  k > 2*MAXCOL-1, IFLAG = 0                +
C                     MAXCOL                                                   +
C                                                                              +
C                     (k-2*MAXCOL)                                             +
C                    s            IF  k > 2*MAXCOL,   IFLAG = 1                +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                    - IF IFLAG = 0                                            +
C                                                                              +
C                     (0)  (0)  (0)  (0)      (0)     (1)     (2)              +
C                    r  , s  , r  , s  ,..., r     , r     , r     , ...       +
C                     1    1    2    2        MAXCOL  MAXCOL  MAXCOL           +
C                                                                              +
C                    - IF IFLAG = 1                                            +
C                                                                              +
C                     (0)  (0)  (0)  (0)      (0)     (1)     (2)              +
C                    r  , s  , r  , s  ,..., s     , s     , s     , ...       +
C                     1    1    2    2        MAXCOL  MAXCOL  MAXCOL           +
C                                                                              +
C           IFLAG  - INPUT INTEGER FLAG.                                       +
C                    IF IFLAG .EQ. 0 THE SUBROUTINE, FOR k >= 2*MAXCOL-1,      +
C                                    RETURNS THE COLUMN MAXCOL OF THE r        +
C                                    VALUES.                                   +
C                    IF IFLAG .EQ. 1 THE SUBROUTINE, FOR k >= 2*MAXCOL,        +
C                                    RETURNS THE COLUMN MAXCOL OF THE s        +
C                                    VALUES.                                   +
C                                                                              +
C           IER    - INPUT/OUTPUT INDEX WARNING/ERROR.                         +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 400     WARNING. CANCELLATION ERROR. NUMERICAL      +
C                                  INSTABILITY PRESENT. THE USER MUST SET      +
C                                  AGAIN IER=0 IN THE CALLING PROGRAM TO       +
C                                  CONTINUE.                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION AND THE VECTOR RSVEC MUST NOT BE MODIFIED    +
C          BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.        +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF r   (J <= MAXCOL) THE USER MUST CALL THIS    +
C                                  J                         (0)               +
C          SUBROUTINE (2*J-1) TIMES. TO OBTAIN THE VALUE OF s   THE USER MUST  +
C          CALL THIS SUBROUTINE 2*J TIMES.                   J                 +
C       -  IF MAXCOL IS EQUAL TO ZERO THEN THE SUBROUTINE RETURNS THE SEQUENCE +
C                                                                              +
C                     (0)  (1)  (2)  (3)                                       +
C                    r  , r  , r  , r  ,  ...                                  +
C                     1    1    1    2                                         +
C                                                                              +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSION FOR THE              +
C          VECTOR ARGUMENT IS:                                                 +
C           RSVEC(0:2*(MAXCOL-1)) IF IFLAG = 0                                 +
C                OR                                                            +
C           RSVEC(0:2*MAXCOL-1)   IF IFLAG = 1                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE RSALGO (RSVEC, MAXCOL, INIT, EPS, RSINIT, IFLAG, IER)       
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, IFLAG, INIT, MAXCOL
      DOUBLE PRECISION EPS, RSINIT
      DOUBLE PRECISION RSVEC(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER IND, INDM, INDS, MAXC2
      DOUBLE PRECISION AUX0, AUX1, AUX2 
      SAVE INDM, MAXC2
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER      = 0                                                           
         INIT     = 1
         INDM     = 0
         RSVEC(0) = RSINIT
         MAXC2    = 2*(MAXCOL - 1)
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C
      IF (MAXCOL .EQ. 0) THEN
         RSVEC(0) = RSINIT
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1).
C
      INDM = INDM + 1 
C
C                        ... IF INDM > 2*(MAXCOL-1) AND IFLAG = 0 THEN 
C                                         (NUMBER_OF_CALLS-2*MAXCOL+1)
C                            THE ELEMENT r 
C                                         MAXCOL
C                            IS COMPUTED.
C                        ... IF INDM > 2*(MAXCOL-1) AND IFLAG = 1 THEN 
C                                         (NUMBER_OF_CALLS-2*MAXCOL)
C                            THE ELEMENT s 
C                                         MAXCOL
C                            IS COMPUTED.
C
      IF (INDM .LE. MAXC2) THEN
         IND = INDM
      ELSE IF (IFLAG .EQ. 0) THEN
         IND = MAXC2 
      ELSE
         IND = MAXC2 + 1
      END IF
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE 
C                            AUX1.
C
      AUX1   = RSINIT
C  
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS.
C
      DO 10 INDS = 0, IND - 1
C  
C                        ... COMPUTE THE VALUE OF THE ELEMENT.
C
         IF (DABS(RSVEC(INDS)) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         AUX0 = (AUX1 / RSVEC(INDS) - 1.0D0)
         IF (DABS(AUX0) .LT. EPS) IER = 400
         IF (INDS .NE. 0) AUX0 = RSVEC(INDS-1) * AUX0
C  
C                        ... SHIFT THE RHOMBUS.
C
         IF (INDS .NE. 0) RSVEC(INDS-1) = AUX2
         AUX2 = AUX1
         AUX1 = AUX0
C  
C                        ... END LOOP FOR INDS.
C
   10 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES.
C
      RSVEC(IND-1) = AUX2
      RSVEC(IND)   = AUX1
C  
C                        ... SET THE RESULT VALUE.
C
      RSINIT = RSVEC(IND)
      RETURN
      END
