C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - THETA                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE THETA-ALGORITHM                        +
C                                                                              +
C   USAGE:                                                                     +
C           CALL THETA (THVEC, MAXCOL, INIT, EPS, THINIT, IER)                 +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           THVEC -  OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF THVEC MUST BE (0:IE), WITH IE GREATER        +
C                    OR EQUAL TO 2*MAXCOL. THIS VECTOR CONTAINS, AFTER         +
C                    THE k-TH CALL:                                            +
C                                                                              +
C                     - IF k = 1                                               +
C                                                                              +
C                            (0)                                               +
C                       theta                                                  +
C                            0                                                 +
C                                                                              +
C                     - IF 1 < k < 3*MAXCOL/2+1 AND MOD(k+1,3)=0               +
C                                                                              +
C                            (k-2)     (k-1)     (k-3)    (k-2)                +
C                       theta  ,  theta  ,  theta  ,  theta       ,...,        +
C                            0         0         1         1                   +
C                                               (0)                            +
C                                0.0D0 ,   theta                               +
C                                               (2*k-1)/3                      +
C                                                                              +
C                     - IF 1 < k < 3*MAXCOL/2+1 AND MOD(k,3)=0                 +
C                                                                              +
C                            (k-2)     (k-1)     (k-3)    (k-2)                +
C                       theta  ,  theta  ,  theta  ,  theta       ,...,        +
C                            0         0         1         1                   +
C                                     (0)              (1)                     +
C                                theta        ,   theta                        +
C                                     2*k/3-1          2*k/3-1                 +
C                                                                              +
C                     - IF 1 < k < 3*MAXCOL/2+1 AND MOD(k-1,3)=0               +
C                                                                              +
C                            (k-2)     (k-1)     (k-3)    (k-2)                +
C                       theta  ,  theta  ,  theta  ,  theta       ,...,        +
C                            0         0         1         1                   +
C                                               (0)                            +
C                                0.0D0 ,   theta                               +
C                                               2*(k-1)/3                      +
C                                                                              +
C                     - IF k >= 3*MAXCOL/2+1                                   +
C                                                                              +
C                            (k-2)     (k-1)     (k-3)    (k-2)                +
C                       theta  ,  theta  ,  theta  ,  theta       ,...,        +
C                            0         0         1         1                   +
C                                    (k-3*MAXCOL/2-1)                          +
C                                theta                                         +
C                                     MAXCOL                                   +
C                                                                              +
C           MAXCOL - INPUT/OUTPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN  +
C                    OF THE THETA SCHEME THAT THE USER WANTS TO COMPUTE        +
C                                            (n)                               +
C                    (THE LOWER INDEX OF theta  ). IF MAXCOL IS ODD THE        +
C                                             m                                +
C                    SUBROUTINE CHANGES ITS VALUE TO (MAXCOL-1).               +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           THINIT - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                         (0)                                                  +
C                    theta          IF k <= 3*MAXCOL/2+1 AND MOD(k-1,3)=0      +
C                         2*(k-1)/3                                            +
C                                                                              +
C                         (1)                                                  +
C                    theta          IF k <  3*MAXCOL/2+1 AND MOD(k+1,3)=0      +
C                         (2*k-1)/3-1                                          +
C                                                                              +
C                         (2)                                                  +
C                    theta          IF k <  3*MAXCOL/2+1 AND MOD(k,3)=0        +
C                         2*k/3-2                                              +
C                                                                              +
C                         (k-3*MAXCOL/2-1)                                     +
C                    theta          IF k > 3*MAXCOL/2+1                        +
C                         MAXCOL                                               +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                         (0)     (1)     (2)     (0)     (1)     (2)          +
C                    theta , theta , theta , theta , theta , theta , ...       +
C                         0       0       0       2       2       2            +
C                                                                              +
C                                 (0)        (1)        (2)        (3)         +
C                            theta   ,  theta    , theta    , theta , ...      +
C                                 MAXCOL     MAXCOL     MAXCOL     MAXCOL      +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION AND THE VECTOR THVEC MUST NOT BE MODIFIED    +
C          BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.        +
C                                      (0)                                     +
C       -  TO OBTAIN THE VALUE OF theta    (2*J <= MAXCOL), THE USER MUST CALL +
C                                      2*J                                     +
C          THIS SUBROUTINE (3*J+1) TIMES.                                      +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENT IS:                                                        +
C           THVEC(0:2*MAXCOL)                                                  +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE THETA (THVEC, MAXCOL, INIT, EPS, THINIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, MAXCOL
      DOUBLE PRECISION EPS, THINIT
      DOUBLE PRECISION THVEC(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER IND, IND2, INDM, INDS, MAXC32
      DOUBLE PRECISION AUX0, AUX1, AUX2, TMP
C
      SAVE INDM, MAXC32
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER      = 0
         INIT     = 1
         INDM     = 0
         THVEC(0) = THINIT
C
C                        ... IF MAXCOL IS ODD IT IS MODIFIED TO 
C                            (MAXCOL - 1).
C
         IF (MOD(MAXCOL,2) .NE. 0) MAXCOL = MAXCOL-1
C
         MAXC32  = 3 * MAXCOL/2 + 1
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         THVEC(0) = THINIT
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1)
C
      INDM = INDM + 1 
C
C                        ... SET THE RESULT VALUE IF INDM = 1
C
      IF (INDM .EQ. 1) THEN
         THVEC(1) = THINIT
         TMP      = THINIT - THVEC(0)
         IF (DABS(TMP) .LT. EPS) THEN
            IER   = 100
            RETURN
         END IF
         THVEC(3) = 1.0D0 / TMP
         RETURN
      END IF
C
C                        ... IF INDM >= (3 * MAXCOL/2 + 1) THEN THE
C                                         (NUMBER_OF_CALLS-3*MAXCOL/2-1)
C                            ELEMENT theta 
C                                         MAXCOL
C                            IS COMPUTED.
C
      IF (INDM .GE. MAXC32) THEN
         IND = MAXCOL
      ELSE IF (MOD(INDM,3) .EQ. 0) THEN
         IND = 2 * INDM/3 
      ELSE
         IND = 2 * INT(INDM/3) + 1
      END IF
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE
C                            AUX1.
C
      AUX1 = THINIT
C
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS
C                            OF THVEC.
C
      DO 10 INDS = 0, IND - 1
         IND2 = 2 * (INDS-1)
         IF (MOD(INDS,2) .EQ. 0) THEN
C  
C                        ... COMPUTE A TERM OF AN ODD COLUMN.
C
            TMP = AUX1 - THVEC(IND2+3)
            IF (DABS(TMP) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            AUX0 = 1.0D0 / TMP
            IF (INDS .NE. 0) THEN
               AUX0 = AUX0 + THVEC(IND2)
C  
C                        ... SHIFT THE ELEMENTS OF THE theta VECTOR.
C
               THVEC(IND2)   = THVEC(IND2+1)
               THVEC(IND2+1) = AUX2
            END IF
C  
C                        ... SHIFT THE RHOMBUS.
C
            AUX2 = AUX1
            AUX1 = AUX0
         ELSE
C  
C                        ... COMPUTE A TERM OF AN EVEN COLUMN.
C
            TMP    = AUX1 - 2 * THVEC(IND2+3) + THVEC(IND2+2)
            IF (DABS(TMP) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            AUX0   = THVEC(IND2) + (THVEC(IND2+1)-THVEC(IND2))*
     *             (AUX1 - THVEC(IND2+3)) / TMP
C  
C                        ... SHIFT THE ELEMENTS OF THE theta VECTOR.
C
            THVEC(IND2)   = THVEC(IND2+1)
            THVEC(IND2+1) = AUX2
C  
C                        ... SHIFT THE RHOMBUS.
C
            AUX2 = AUX1
            AUX1 = AUX0
         END IF
C
C                        ... END LOOP FOR INDS.
C
   10 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES, SHIFT OF
C                            THE LAST ELEMENTS OF THE theta VECTOR
C                            AND SET THE RESULT VALUE.
C
      IND2 = IND * 2 
      THVEC(IND2-2)  = THVEC(IND2-1)
      THVEC(IND2-1)  = AUX2
      IF (IND .EQ. MAXCOL) THEN
         THVEC(IND2) = AUX1
         THINIT      = AUX1
      ELSE 
         IF (MOD(INDM,3) .NE. 0) THEN
            THVEC(IND2) = THVEC(IND2+1)
            THINIT      = THVEC(IND2-1)
         ELSE
            THINIT      = AUX1
         END IF
         THVEC(IND2+1)  = AUX1
      END IF
      RETURN
      END
